#!/usr/bin/python

###############################################################################
#    SVdB/BreakDB -    A database application for storing structural varations
#		and their annotations.
#    Copyright (C) 2008  Philip Cayting
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the Creative Commons (Attribution-NonCommercial); 
#	 version 2.5.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#
#
#Contact:		Philip Cayting, MB&B Department, Yale University, 
#				PO BOX 208114, New Haven, CT 06520-8114
#				philip.cayting@yale.edu
###############################################################################

import MySQLdb.cursors
import SVEvent
import datetime

def getConnection():
	return MySQLdb.connect (host="mysql.gersteinlab.dreamhosters.com",
						user="gersteinlab",
						passwd="viewer",
						db="svdb")

def getEventIDs(index, range, conn, source_id=None):
	cursor = conn.cursor()
	total_count = 0
	if source_id:
		sql = "SELECT count(*) FROM variant_to_source WHERE source_id = '%i'" % source_id
		cursor.execute(sql)
		total_count = cursor.fetchone()[0]
		sql = "SELECT a.variant_id FROM variant_to_source vs, archive a WHERE vs.source_id = '%i' AND a.variant_table_id = vs.variant_table_id ORDER BY a.variant_id LIMIT %i OFFSET %i" % (source_id, range, index)
	else:
		sql = "SELECT count(*) FROM current_variant"
		cursor.execute(sql)
		total_count = cursor.fetchone()[0]
		sql = "SELECT variant_id FROM current_variant ORDER BY id LIMIT " + str(range) + " OFFSET " + str(index)
	cursor.execute(sql)
	ids = []
	for result in cursor.fetchall():
		ids.append(result[0])
	return (ids, total_count)
			
def getValidMechanisms(conn):
	cursor = conn.cursor()
	sql = "SELECT name FROM event_class ORDER BY name ASC"
	cursor.execute(sql)
	event_classes = []
	for result in cursor.fetchall():
		event_classes.append(result[0])
	return event_classes
	
def getValidEventClasses(conn):
	cursor = conn.cursor()
	sql = "SELECT name FROM variant_class ORDER BY name ASC"
	cursor.execute(sql)
	classes = []
	for result in cursor.fetchall():
		classes.append(result[0])
	return classes
	
def allSpecies(conn):
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT * FROM species ORDER BY name ASC"
	cursor.execute(sql)
	species = []
	for result in cursor.fetchall():
		s = Species(result['name'], result['build'], result['tax_id'], result['species_id'])
		species.append(s)
	return species
	
def getSpecies(species_id, conn):
	species_id = int(species_id)
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT * FROM species WHERE species_id = '%i'" % species_id
	cursor.execute(sql)
	result = cursor.fetchone()
	return Species(result['name'], result['build'], result['tax_id'], result['species_id'])
	
def getReads(event_id, conn):
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT * FROM genome_read WHERE variant_id = '%s'" % event_id
	cursor.execute(sql)
	reads = []
	for result in cursor.fetchall():
		r_id = result['genome_read_id']
		e_id = result['variant_id']
		if result.has_key('read_name'):
			r_name = result['read_name']
		if result.has_key('chromosome') and result['chromosome']:
			chr = result['chromosome']
		else:
			chr = None
		if result.has_key('start_coord') and result['start_coord']:
			start = int(result['start_coord'])
		else:
			start = None
		if result.has_key('stop_coord') and result['stop_coord']:
			stop = result['stop_coord']
		else:
			stop = None
		if result.has_key('strand') and result['strand']:
			strand = result['strand']
		else:
			strand = None
		if result.has_key('sequence') and result['sequence']:
			seq = result['sequence']
		else:
			seq = None
		reads.append( Read(r_id, e_id, r_name, chr, start, stop, strand, seq) )
	return reads
		
def getReadPair(read_pair_id, conn):
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT * from genome_read_pairs WHERE id = '%i'" % read_pair_id
	cursor.execute(sql)
	result = cursor.fetchone()
	if not result:  return None
	rp_id = result['id']
	left_read = Read(read_id=result['left_read_id'])
	left_read.load(conn)
	right_read = Read(read_id=result['right_read_id'])
	right_read.load(conn)
	event_id = result['variant_id']
	if result.has_key('name'):
		rp_name = result['name']
	else:
		rp_name = None
	rp = ReadPair(left_read=left_read, right_read=right_read, name=rp_name, event_id=event_id, read_pair_id=rp_id)
	return rp
	
def getReadPairs(event_id, conn):
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT id from genome_read_pairs WHERE variant_id = '%s'" % event_id
	cursor.execute(sql)
	read_pairs = []
	for result in cursor.fetchall():
		read_pairs.append(getReadPair(result['id'], conn))
	return read_pairs

def getAllSources(conn):
	'''Returns all sources as (source name, source id, source count) tuples'''
	cursor = conn.cursor()
	sql = "SELECT COUNT(vs.variant_table_id), vs.source_id, s.name FROM variant_to_source vs, source s WHERE s.source_id = vs.source_id GROUP BY source_id"
	cursor.execute(sql)
	sources = []
	for result in cursor.fetchall():
		if not result: return []
		sources.append((result[2], result[1], result[0]))
	return sources

def getSource(id, conn):
	cursor = conn.cursor(MySQLdb.cursors.DictCursor)
	sql = "SELECT * FROM source WHERE source_id = '%i'" % id
	cursor.execute(sql)
	result = cursor.fetchone()
	if not result: return None
	return (result['name'], int(result['source_id']))
	
class ReadPair:
	def __init__(self, left_read, right_read, name=None, event_id=None, read_pair_id=None):
		self.left_read = left_read
		self.right_read = right_read
		self.name = name
		self.event_id = event_id
		self.read_pair_id = read_pair_id
		
	def commit(self, conn):
		cursor = conn.cursor()
		sql = "INSERT INTO genome_read_pairs SET left_read_id = '%i', right_read_id = '%i', variant_id = '%s'" % (self.left_read.read_id, self.right_read.read_id, self.event_id)
		if self.name:
			sql += ", name = '%s'" % self.name
		cursor.execute(sql)
		sql = "SELECT id FROM genome_read_pairs WHERE left_read_id='%i' AND right_read_id='%i' AND variant_id='%s'" % (self.left_read.read_id, self.right_read.read_id, self.event_id)
		if self.name:
			sql += " AND name='%s'" % self.name
		sql += " ORDER BY id DESC"
		cursor.execute(sql)
		return cursor.fetchone()[0]
		
		
class Read:
	def __init__(self, read_id=None, event_id=None, read_name=None, chr=None, start_coord=None, stop_coord=None, strand=None, sequence=None):
		self.read_id = read_id
		self.read_name = read_name
		self.event_id = event_id
		self.chr = chr
		self.start_coord = start_coord
		self.stop_coord = stop_coord
		self.strand = strand
		self.sequence = sequence
		
	def commit(self, conn):
		cursor = conn.cursor()
		sql = "INSERT INTO genome_read SET read_name = '%s', variant_id = '%s', chromosome = '%s', start_coord = '%i', stop_coord = '%i', sequence = '%s', strand = '%s'" % (self.read_name, self.event_id, self.chr, self.start_coord, self.stop_coord, self.sequence, self.strand)
		cursor.execute(sql)
		sql = "SELECT genome_read_id WHERE read_name='%s' AND variant_id='%s' AND chromosome='%s' AND start_coord='%i' AND stop_coord='%i' AND sequence='%s' AND strand='%s' ORDER BY genome_read_id DESC" % (self.read_name, self.event_id, self.chr, self.start_coord, self.stop_coord, self.sequence, self.strand)
		cursor.execute(sql)
		return cursor.fetchone()[0]
		
	def load(self, conn):
		assert self.read_id is not None
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		sql = "SELECT * from genome_read WHERE genome_read_id='%i'" % self.read_id
		cursor.execute(sql)
		result = cursor.fetchone()
		self.event_id = result['variant_id']
		if result.has_key('read_name'):
			self.read_name = result['read_name']
		if result.has_key('chromosome') and result['chromosome']:
			self.chr = result['chromosome']
		if result.has_key('start_coord') and result['start_coord']:
			self.start_coord = int(result['start_coord'])
		if result.has_key('stop_coord') and result['stop_coord']:
			self.stop_coord = int(result['stop_coord'])
		if result.has_key('strand') and result['strand']:
			self.strand = result['strand']
		if result.has_key('sequence') and result['sequence']:
			self.sequence = result['sequence']
		
	def __str__(self):
		return self.read_id
		
	def coordString(self):
		strand = '-'
		if self.strand: strand = self.strand
		coord = "chr%s:%i%s%i" % (self.chr, self.start_coord, strand, self.stop_coord)
		return coord
	
class Species:
	ucsc_mapping = { '36':'hg18', '35':'hg17', '34':'hg16' }
	def __init__(self, name, build, tax_id, species_id=0):
		self.name = name
		self.build = build
		self.tax_id = tax_id
		self.species_id = species_id
		
	def ucsc_build(self):
		return self.ucsc_mapping[self.build]
		
	def display(self):
		return "%s (Build: %s)" % (self.name, self.build)
		
	def __str__(self):
		return "%s (Build: %s)" % (self.name, self.build)
	
class EventCoordinates:
	def __init__(self, start_coordinate, stop_coordinate, context, chromosome):
		self.start = int(start_coordinate)
		self.stop = int(stop_coordinate)
		self.context = context
		if chromosome and not isinstance(self.context, SVEvent.SVEvent):
			self.chr = chromosome
		else:
			self.chr = None
					
	def compare(self, secondary):
		if not (self.start == secondary.start and self.stop == secondary.stop):
			return False
		if not self.context == secondary.context:
			return False
		if not self.chr == secondary.chr:
			return False			
		# Else:
		return True
		
	def __str__(self):
		if isinstance(self.context, SVEvent.SVEvent):
			return "%s:%i-%i" % (self.context.event_id, self.start, self.stop)
		else:
			return "chr%s:%i-%i" % (self.chr, self.start, self.stop)
		
class Evidence:
	def __init__(self, primate_sequence, homology_length, homology_percentage):
		if primate_sequence: self.primate_sequence = True
		else: self.primate_sequence = False
		if homology_length: self.homology_length = homology_length
		else: self.homology_length = None
		if homology_percentage: self.homology_percentage = homology_percentage
		else:  self.homology_percentage = None
		
	def display(self):
		display = ""
		if self.primate_sequence:
			display += "Sequence Present in Primates<BR>"
		if self.homology_length:
			display += "Homology Length: " + str(self.homology_length) + "<BR>"
		if self.homology_percentage:
			display += "Homology Percentage: " + str(self.homology_percentage) + "<BR>"			
		if display == "": display = "--"
		return display
		
	def __str__(self):
		display = ""
		if self.primate_sequence:
			display += "Sequence Present in Primates  "
		if self.homology_length:
			display += "Homology Length: " + str(self.homology_length) + "  "
		if self.homology_percentage:
			display += "Homology Percentage: " + str(self.homology_percentage) + "  "			
		return display
		
class InsertSVEvent:
	def __init__(self, event):
		self.event_id = event.event_id
		self.event_class = event.event_class
		self.event_coordinates = event.event_coordinates
		self.mechanism = event.mechanism
		self.evidence = event.evidence
		self.confidence_level = event.confidence_level
		self.confidence_description = event.confidence_description
		self.mechanism_notes = event.mechanism_notes
		self.left_sequence = event.left_sequence
		self.right_sequence = event.right_sequence
		self.evidence_notes = event.evidence_notes
		self.validation_notes = event.validation_notes
		self.date_created = datetime.date.today().isoformat()
		self.related_events = event.related_events
		self.sources = event.sources
		self.species = event.species
		self.inserted_sequence = event.inserted_sequence
		self.read_pairs = event.read_pairs
		
	def dbCommit(self, conn):
		cursor = conn.cursor()
		sql = "SELECT MAX(archive_id) FROM archive WHERE variant_id = '" + str(self.event_id) + "'"
		cursor.execute(sql)
		max_archive = cursor.fetchone()
		if max_archive and max_archive[0] and max_archive[0] != 0: 
			self.archive_number = max_archive[0] + 1
		else: 
			self.archive_number = 1
			self.event_id = self.getNextVariantID(conn)	
		coordinates_id = self.commitVariantCoordinates(self.event_coordinates, conn)
		evidence_id = self.commitEvidence(self.evidence, conn)
		variant_class_id = self.getVariantClassID(self.event_class, conn)
		event_class_id = self.getEventClassID(self.mechanism, conn)
		variant_table_id = self.getNextVariantTableID(conn)
		self.commitSources(variant_table_id, conn)
		self.commitRelatedVariants(variant_table_id, conn)
		
		insert_sql = "INSERT INTO variant SET variant_table_id = " + str(variant_table_id) + " "
		if variant_class_id: 
			insert_sql += ", variant_class_id = " + str(variant_class_id) + " "
		if coordinates_id:
			insert_sql += ", variant_coordinates_id = " + str(coordinates_id) + " " 
		if evidence_id:
			insert_sql += ", evidence_id = " + str(evidence_id) + " "
		if event_class_id:
			insert_sql += ", event_class_id = " + str(event_class_id) + " "
		if self.confidence_level:
			insert_sql += ", confidence_level = '" + str(self.confidence_level) + "' "
		if self.confidence_description:
			insert_sql += ", confidence_description = '" + str(self.confidence_description) + "' "
		if self.mechanism_notes:
			insert_sql += ", event_notes = '" + self.mechanism_notes + "' "
		if self.left_sequence:
			insert_sql += ", left_sequence = '" + self.left_sequence + "' "
		if self.right_sequence:
			insert_sql += ", right_sequence = '" + self.right_sequence + "' "
		if self.evidence_notes:
			insert_sql += ", evidence_notes = '" + self.evidence_notes + "' "
		if self.validation_notes:
			insert_sql += ", validation_notes = '" + self.validation_notes + "' "
		if self.date_created:
			insert_sql += ", date_created = '" + self.date_created + "' "
		if self.species:
			insert_sql += ", species_id = ' " + str(self.species.species_id) + "' "
		if self.inserted_sequence:
			insert_sql += ", inserted_sequence = ' " + str(self.inserted_sequence) + "' "
		
		
		cursor.execute(insert_sql)
		
		# Add Event to Archive Table
		insert_sql = "INSERT INTO archive VALUES(" + str(self.archive_number) + ", '" + str(self.event_id) + "', " + str(variant_table_id) + ")"
		cursor.execute(insert_sql)
		
		# Update Current Variant Table
		if self.archive_number > 1:
			update_sql = "UPDATE current_variant SET archive_id = " + str(self.archive_number) + " WHERE variant_id = '" + str(self.event_id) + "'"
			cursor.execute(update_sql)
		else:
			insert_sql = "INSERT INTO current_variant SET archive_id = " + str(self.archive_number) + ", variant_id = '" + str(self.event_id) + "'"
			cursor.execute(insert_sql)
		
		# Add Reads
		for rp in self.read_pairs:
			rp.left_read.event_id = self.event_id
			rp.right_read.event_id = self.event_id
			rp.event_id = self.event_id
			rp.left_read.read_id = rp.left_read.commit(conn)
			rp.right_read.read_id = rp.right_read.commit(conn)
			rp.id = rp.commit(conn)
			
		return self.event_id
				
	def getNextVariantID(self, conn):
		cursor = conn.cursor()
		cursor.execute("SELECT MAX(id) FROM current_variant")
		r = cursor.fetchone()
		if not r or not r[0]: return "VX1"
		else: return "VX" + str(r[0] + 1)
			
	def commitVariantCoordinates(self, coordinates, conn):
		vc_dict = { 'start_coordinate' : coordinates.start,
				    'stop_coordinate' : coordinates.stop,
				  }
				  
		if coordinates.chr:
			vc_dict['chromosome'] = coordinates.chr
			vc_dict['reference_context'] = coordinates.context
		else:
			vc_dict['context_id'] = coordinates.context.event_id
			vc_dict['reference_context'] = 'no'

		return IndexedObject('variant_coordinates', vc_dict).commit(conn)
		
	def commitEvidence(self, evidence, conn):
		if not evidence: return None
		
		ev_dict = {}
		if evidence.homology_length: 
			ev_dict['homology_length'] = evidence.homology_length
		if evidence.homology_percentage:
			ev_dict['homology_percentage'] = evidence.homology_percentage
		if evidence.primate_sequence:
			ev_dict['primate_sequence'] = 'y'
			
		return IndexedObject('evidence', ev_dict).commit(conn)
		
	def commitSources(self, variant_table_id, conn):
		if not self.sources or len(self.sources) < 1:
			return []
		cursor = conn.cursor()
		source_id_list = []
		for source in self.sources:
			# Determine if source already exists
			sql = "SELECT source_id FROM source WHERE name='%s' AND pubmed_id='%s'" % (source['name'], source['pubmed_id'])
			cursor.execute(sql)
			result = cursor.fetchone()
			if result:
				source_id_list.append(result[0])
			else:
				# Source doesn't exist, add it
				sql = "INSERT INTO source SET name='%s', pubmed_id='%s'" % (source['name'], source['pubmed_id'])
				cursor.execute(sql)
				# Get Source ID
				sql = "SELECT source_id FROM source WHERE name='%s' AND pubmed_id='%s'" % (source['name'], source['pubmed_id'])
				cursor.execute(sql)
				source_id_list.append(cursor.fetchone()[0])
		for source_id in source_id_list:
			sql = "INSERT INTO variant_to_source SET variant_table_id='%s', source_id='%s'" % (variant_table_id, source_id)
			cursor.execute(sql)
		return source_id_list
		
	def commitRelatedVariants(self, variant_table_id, conn):
		if not self.related_events or len(self.related_events) < 1:
			return []
		ids = []
		cursor = conn.cursor()
		for event in self.related_events:
			sql = "INSERT INTO related_variant SET parent_variant_table_id='%s', child_variant_id='%s'" % (variant_table_id, event.event_id)
			cursor.execute(sql)
			ids.append(event.event_id)
		return ids
		
	def getVariantClassID(self, variant_class_name, conn):
		if not variant_class_name: return None
		
		return ListObject('variant_class').getID(variant_class_name, conn)
		
	def getEventClassID(self, event_class_name, conn):
		if not event_class_name: return None
		
		return ListObject('event_class').getID(event_class_name, conn)
		
	def getNextVariantTableID(self, conn):
		cursor = conn.cursor()
		cursor.execute("SELECT MAX(variant_table_id) FROM variant")
		r = cursor.fetchone()
		if not r or not r[0]: return 1
		else: return r[0] + 1
		
class DBActiveSVEvent:
	def __init__(self, id, conn):
		self.event_id = id
		#print self.event_id
		
		# Get Variant Table Information
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		sql = "SELECT v.* FROM current_variant cv, archive a, variant v WHERE cv.variant_id = '" + self.event_id + "' AND a.variant_id = cv.variant_id AND a.archive_id = cv.archive_id AND v.variant_table_id = a.variant_table_id"
		cursor.execute(sql)
		sql_variant = cursor.fetchone()
		
		self.event_class = self.getVariantClass(sql_variant["variant_class_id"], conn)
		self.event_coordinates = self.getVariantCoordinates(sql_variant["variant_coordinates_id"], conn)
		self.archives = self.getVariantArchives(self.event_id, conn)
		self.mechanism = self.getEventClass(sql_variant["event_class_id"], conn)
		self.evidence = self.getEvidence(sql_variant["evidence_id"], conn)
		self.archive_number = self.getArchiveNumber(self.event_id, conn)
		self.date_created = sql_variant["date_created"]
		self.sources = self.getSources(int(sql_variant["variant_table_id"]), conn)
		self.related_events = self.getRelatedVariants(int(sql_variant["variant_table_id"]), conn)
		self.species = self.getSpecies(int(sql_variant["species_id"]), conn)
		self.collectSimpleFields(sql_variant)
		
	def convertSQLDate(self, sql_date):
		if not sql_date:  return None
		
		date_str = sql_date.split(' ')[0]
		return date_str
			
	def collectSimpleFields(self, sql_variant):		
		# Simple Table Information		
		if sql_variant["confidence_level"]:
			self.confidence_level = sql_variant["confidence_level"]
		else:
			self.confidence_level = None
		if sql_variant["confidence_description"]:
			self.confidence_description = sql_variant["confidence_description"]
		else:
			self.confidence_description = None
		if sql_variant["event_notes"]:
			self.mechanism_notes = sql_variant["event_notes"]
		else:
			self.mechanism_notes = None
		if sql_variant["left_sequence"]:
			self.left_sequence = sql_variant["left_sequence"]
		else:
			self.left_sequence = None
		if sql_variant["right_sequence"]:
			self.right_sequence = sql_variant["right_sequence"]
		else:
			self.right_sequence = None
		if sql_variant["evidence_notes"]:
			self.evidence_notes = sql_variant["evidence_notes"]
		else:
			self.evidence_notes = None
		if sql_variant["validation_notes"]:
			self.validation_notes = sql_variant["validation_notes"]
		else:
			self.validation_notes = None
		if sql_variant["inserted_sequence"]:
			self.inserted_sequence = sql_variant["inserted_sequence"]
		else:
			self.inserted_sequence = None
			
	def getSpecies(self, species_id, conn):
		if not species_id: return None		
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		cursor.execute("SELECT * FROM species WHERE species_id = '%i'" % species_id)
		result = cursor.fetchone()
		if not result: return None
		else: return Species(result['name'], result['build'], result['tax_id'] ,result['species_id'])
			
	def getVariantClass(self, variant_class_id, conn):
		if not variant_class_id: return None
		cursor = conn.cursor()
		cursor.execute("SELECT name FROM variant_class WHERE variant_class_id = '" + str(variant_class_id) + "'")
		return cursor.fetchone()[0]
		
	def getVariantCoordinates(self, variant_coordinates_id, conn):
		if not variant_coordinates_id: return None
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		cursor.execute("SELECT * FROM variant_coordinates WHERE variant_coordinates_id = '" + str(variant_coordinates_id) + "'")
		r = cursor.fetchone()
		
		# Assign Context (either another variant or the reference genome)
		context = None
		if r["context_id"] and r["context_id"] != 'no': 
			context = SVEvent.ExistingSVEvent(r["context_id"], conn)
		else: context = r["reference_context"]		
		return EventCoordinates( start_coordinate = r["start_coordinate"],
								   stop_coordinate = r["stop_coordinate"],
								   context = context,
								   chromosome = r["chromosome"] )
								   
	def getVariantArchives(self, variant_id, conn):
		if not variant_id: return None	
		cursor = conn.cursor()
		cursor.execute("SELECT a.archive_id FROM archive a, variant v WHERE a.variant_id = '" + str(variant_id) + "' and a.variant_table_id = v.variant_table_id ORDER BY v.date_created DESC, a.archive_id DESC")
		
		archives = []
		for result in cursor.fetchall():
			archive_number = result[0]
			archives.append( SVEvent.ArchiveSVEvent(variant_id, archive_number, conn) )		
		return archives
		
	def getEventClass(self, event_class_id, conn):
		if not event_class_id: return None		
		cursor = conn.cursor()
		cursor.execute("SELECT name FROM event_class WHERE event_class_id = '" + str(event_class_id) + "'")		
		return cursor.fetchone()[0]
		
	def getEvidence(self, evidence_id, conn):
		if not evidence_id: return None		
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		cursor.execute("SELECT * FROM evidence WHERE evidence_id = '" + str(evidence_id) + "'")
		r = cursor.fetchone()	
		return Evidence( primate_sequence=r["primate_sequence"],
						 homology_length=r["homology_length"],
						 homology_percentage=r["homology_percentage"] )
						 
	def getArchiveNumber(self, variant_id, conn):
		cursor = conn.cursor()
		cursor.execute("SELECT archive_id FROM current_variant WHERE variant_id = '" + str(variant_id) + "'")
		return cursor.fetchone()[0]
		
	def getSources(self, variant_table_id, conn):
		if not variant_table_id: return None
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		cursor.execute("SELECT s.* FROM source s, variant_to_source vs WHERE vs.variant_table_id = '%i' AND vs.source_id = s.source_id" % variant_table_id)
		source_list = []
		for sql_source in cursor.fetchall():
			source_list.append(sql_source)
		return source_list
		
	def getRelatedVariants(self, variant_table_id, conn):
		if not variant_table_id: return None
		cursor = conn.cursor()
		cursor.execute("SELECT child_variant_id FROM related_variant WHERE parent_variant_table_id = '%i'" % variant_table_id)
		related_list = []
		for sql_relation in cursor.fetchall():
			if sql_relation[0] == 0: continue
			id = sql_relation[0]
			related_list.append(id)
		return related_list
			
class DBArchiveSVEvent(DBActiveSVEvent):
	def __init__(self, event_id, archive_number, conn):
		self.event_id = event_id
		self.archive_number = archive_number
		# Get Variant Table Information
		cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		sql = "SELECT v.* FROM archive a, variant v WHERE a.archive_id = '" + str(archive_number) + "' AND a.variant_id = '" + str(event_id) + "' AND v.variant_table_id = a.variant_table_id"
		cursor.execute(sql)
		sql_variant = cursor.fetchone()		
		self.event_class = self.getVariantClass(sql_variant["variant_class_id"], conn)
		self.event_coordinates = self.getVariantCoordinates(sql_variant["variant_coordinates_id"], conn)
		self.mechanism = self.getEventClass(sql_variant["event_class_id"], conn)
		self.evidence = self.getEvidence(sql_variant["evidence_id"], conn)
		self.date_created = sql_variant["date_created"]
		self.sources = self.getSources(int(sql_variant["variant_table_id"]), conn)
		self.related_events = self.getRelatedVariants(int(sql_variant["variant_table_id"]), conn)
		self.species = self.getSpecies(int(sql_variant["species_id"]), conn)
		
		self.collectSimpleFields(sql_variant)
				
	def dbCommit(self, conn):
		return None
		
class IndexedObject:
	def __init__(self, table, field_dict):
		''' For tables which are indexed by table_id field
		 Expects field_dict to contain all relevant fields to insert in form:
			field_name : field_value'''		
			
		self.table = table
		self.field_dict = field_dict
		
	def commit(self, conn):
		'Inserts object into appropriate table, returns id'		
		
		id = self.getNextID(conn)
		sql = "INSERT INTO " + self.table + " SET " + self.table + "_id = " + str(id)
		for field_name in self.field_dict.keys():
			sql += ", " + str(field_name) + " = '" + str(self.field_dict[field_name]) + "' "
		cursor = conn.cursor()
		cursor.execute(sql)		
		return id
		
	def getNextID(self, conn):
		'Gets next available ID.  Note, not properly locked (can give same key)'
		
		sql = "SELECT MAX(" + self.table + "_id) FROM " + self.table
		cursor = conn.cursor()
		cursor.execute(sql)
		r = cursor.fetchone()
		if not r or not r[0]: return 1
		else: return r[0] + 1
		
class ListObject:
	def __init__(self, table):
		self.table = table
		
	def getID(self, name, conn):
		cursor = conn.cursor()
		sql = "SELECT " + self.table + "_id FROM " + self.table + " WHERE name = '" + str(name) + "'"
		cursor.execute(sql)
		r = cursor.fetchone()
		if not r: return None
		else: return r[0]
		
	def getName(self, id, conn):
		cursor = conn.cursor()
		sql = "SELECT name FROM " + self.table + " WHERE " + self.table + "_id  = '" + str(id) + "'"
		cursor.execute(sql)
		r = cursor.fetchone()
		if not r: return None
		else: return r[0]
		
	def getAll(self, conn):
		cursor.conn.cursor(MySQLdb.cursors.DictCursor)
		cursor.execute("SELECT * FROM " + self.table)
		objects = []
		for result in cursor.fetchall():
			objects.append([result[self.table+"_id"], result["name"]])
		return objects
