CREATE TABLE `archive` (
  `archive_id` int(11) default NULL,
  `variant_id` varchar(50) default NULL,
  `variant_table_id` int(11) default NULL
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `current_variant` (
  `variant_id` varchar(50) default NULL,
  `archive_id` varchar(50) default NULL,
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `event_class` (
  `event_class_id` int(11) NOT NULL auto_increment,
  `name` varchar(100) default NULL,
  PRIMARY KEY  (`event_class_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `evidence` (
  `evidence_id` int(11) NOT NULL auto_increment,
  `primate_sequence` enum('y','n') default NULL,
  `homology_length` int(11) default NULL,
  `homology_percentage` float default NULL,
  PRIMARY KEY  (`evidence_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `genome_read` (
  `genome_read_id` int(11) NOT NULL auto_increment,
  `variant_id` varchar(50) NOT NULL,
  `chromosome` varchar(2) default NULL,
  `start_coord` int(11) default NULL,
  `stop_coord` int(11) default NULL,
  `sequence` text,
  `strand` enum('+','-') default NULL,
  `read_name` varchar(30) default NULL,
  PRIMARY KEY  (`genome_read_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `genome_read_pairs` (
  `id` int(11) NOT NULL auto_increment,
  `left_read_id` int(11) default NULL,
  `right_read_id` int(11) default NULL,
  `name` varchar(100) default NULL,
  `variant_id` varchar(50) default NULL,
  PRIMARY KEY  (`id`),
  KEY `left_read_id` (`left_read_id`,`right_read_id`),
  KEY `right_read_id` (`right_read_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `related_variant` (
  `parent_variant_table_id` int(11) NOT NULL,
  `child_variant_id` varchar(50) default NULL
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `source` (
  `source_id` int(11) NOT NULL auto_increment,
  `name` varchar(250) default NULL,
  `pubmed_id` varchar(50) default NULL,
  PRIMARY KEY  (`source_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 ;

CREATE TABLE `species` (
  `species_id` int(11) NOT NULL auto_increment,
  `name` varchar(100) NOT NULL,
  `build` varchar(10) NOT NULL,
  `tax_id` varchar(6) NOT NULL,
  PRIMARY KEY  (`species_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `variant` (
  `variant_table_id` int(11) NOT NULL auto_increment,
  `variant_class_id` int(11) default NULL,
  `variant_coordinates_id` int(11) default NULL,
  `confidence_level` float default NULL,
  `confidence_description` text,
  `event_class_id` int(11) default NULL,
  `event_notes` text,
  `evidence_id` int(11) default NULL,
  `left_sequence` text,
  `right_sequence` text,
  `evidence_notes` text,
  `validation_notes` text,
  `date_created` timestamp NOT NULL default CURRENT_TIMESTAMP,
  `species_id` int(11) NOT NULL,
  `inserted_sequence` text,
  PRIMARY KEY  (`variant_table_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `variant_class` (
  `variant_class_id` int(11) NOT NULL auto_increment,
  `name` varchar(100) default NULL,
  PRIMARY KEY  (`variant_class_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `variant_coordinates` (
  `variant_coordinates_id` int(11) NOT NULL auto_increment,
  `context_id` varchar(100) default NULL,
  `reference_context` enum('no','hg18') default 'no',
  `start_coordinate` int(11) default NULL,
  `stop_coordinate` int(11) default NULL,
  `chromosome` varchar(10) default NULL,
  PRIMARY KEY  (`variant_coordinates_id`),
  KEY `context_id` (`context_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `variant_to_source` (
  `variant_table_id` int(11) NOT NULL,
  `source_id` int(11) NOT NULL
) ENGINE=MyISAM DEFAULT CHARSET=latin1;