#!/usr/bin/python

# Author:  	Philip Cayting, Yale University
#			philip.cayting@yale.edu
# Function:  Displays edit page for a given Event and commits those changes 
#  once submitted.

import cgi
import cgitb; cgitb.enable()
from SVEvent import *
import SVdB

VARIANT_CELL_COLOR = "BFD7FE"
EDIT_CELL_COLOR = "FFFFCC"
NUM_SOURCES = 5

conn = SVdB.getConnection()
						
form = cgi.FieldStorage()


def makeSourceList(form):
	source_list = []
	for i in range(0, NUM_SOURCES):
		source_name = form.getfirst("sources_source%i" % i)
		pubmed_id = form.getfirst("sources_pubmed%i" % i)
		if source_name or pubmed_id:
			s = { 'name' : source_name, 'pubmed_id' : pubmed_id }
			source_list.append(s)
	return source_list

def updateSVEvent(id, form, conn):
	
	oldEvent = ExistingSVEvent(id, conn)
	
	event_class = oldEvent.event_class
	if form.has_key("event_class"):
		event_class = form.getfirst("event_class")
	new_context = 'hg18'
	if form.has_key("context_id"):
		new_context = ExistingSVEvent(form.getfirst("context_id"), conn)
	new_coordinates = SVdB.EventCoordinates(
			start_coordinate=int(form.getfirst("start_coordinate")),
			stop_coordinate=int(form.getfirst("stop_coordinate")),
			context=new_context,
			chromosome=form.getfirst("chromosome")
			)
	event_coordinates = oldEvent.event_coordinates
	if not new_coordinates.compare(oldEvent.event_coordinates):
		event_coordinates = new_coordinates
	
	primate_sequence = form.getfirst("primate_sequence")
	homology_length = form.getfirst("homology_length")
	homology_percentage = form.getfirst("homology_percentage")
	
	if homology_length:  homology_length = float(homology_length)
	if homology_percentage:  homology_percentage = float(homology_percentage)
	
	evidence = SVdB.Evidence(
			primate_sequence=primate_sequence,
			homology_length=homology_length,
			homology_percentage=homology_percentage
			)
	if not evidence.primate_sequence and evidence.homology_length and evidence.homology_percentage:
		evidence = None
		
	sources = makeSourceList(form)
	related_events = form.getfirst("related_events")
	if related_events:
		related_events = related_events.replace(' ', '')
		related_events = related_events.split(',')
		related_events = map(lambda x: ExistingSVEvent(x, conn), related_events)

	mechanism = oldEvent.mechanism
	if form.has_key("mechanism"):
		mechanism = form.getfirst("mechanism")
	confidence_level = form.getfirst("confidence_level")
	confidence_description = form.getfirst("confidence_description")
	mechanism_notes = form.getfirst("mechanism_notes")
	left_sequence = form.getfirst("left_sequence")
	right_sequence = form.getfirst("right_sequence")
	evidence_notes = form.getfirst("evidence_notes")
	validation_notes = form.getfirst("validation_notes")
	species = None
	if form.has_key("species"):
		species = SVdB.getSpecies(form.getfirst("species"), conn)
	inserted_sequence = form.getfirst("inserted_sequence")
	
	newSVEvent = NewSVEvent( event_id=id,
							 event_class=event_class,
							 event_coordinates=event_coordinates,
							 mechanism=mechanism,
							 confidence_level=confidence_level,
							 confidence_description=confidence_description,
							 mechanism_notes=mechanism_notes,
							 left_sequence=left_sequence,
							 right_sequence=right_sequence,
							 evidence=evidence,
							 evidence_notes=evidence_notes,
							 validation_notes=validation_notes,
							 sources=sources,
							 related_events=related_events,
							 species=species,
							 inserted_sequence=inserted_sequence)
	SVdB.InsertSVEvent(newSVEvent).dbCommit(conn)
	newSVEvent = ExistingSVEvent(id, conn)
	

	print "Location: display.py?id=%s" % (id)
	print

if form.has_key("event_id"):
	updateSVEvent(form.getfirst("event_id"), form, conn)
elif not form.has_key("id"):
	print "Content-type: text/html"
	print
	print "<img src=\"svdb-header.gif\">"
	print "<font face=\"Arial, Helvetica, sans-serif\">"
	print "Error, no ID specified"
else:
	print "Content-type: text/html"
	print
	
	print "<img src=\"svdb-header.gif\">"
	print "<font face=\"Arial, Helvetica, sans-serif\">"
	print "<H2>Edit Event</H2>"
	
	id = form.getfirst("id")
	
	v = ExistingSVEvent(id, conn)
	
	print "<form action=\"edit.py\" method=\"post\">"
	print "<input type=\"hidden\" name=\"event_id\" value=\"" + id + "\">"
	print v.displayEditHTML(conn, EDIT_CELL_COLOR)
	print "<input type=\"submit\" value=\"Submit Changes\">"