#!/usr/bin/perl -w

use strict;
my $overall_count = 0;
my $min_concordant=1256;
my $max_concordant=7239;
my $mean_concordant = 3015; #NA18505: 2942
my $ignore_MT_DNA_origin = 1;
my $blat_seq_ID_cutoff = 25;
#my $genome_DB="~/Data/Genomes/Ensembl/Homo_sapiens.NCBI35.may/Homo_sapiens.NCBI35.may.dna.fa";
my $genome_DB="Data/Human_Build36_hg18_Mar2006.2bit";
my $ooc_file = "Data/11.ooc"; #disabled!
my $min_read_match_length = 10;
my $CELERA;
my (%PREVIOUSLY_SUCCESSFULLY_EVALUATED, %PREVIOUSLY_EVALUATED);
my $translocations=0;
my $also_report_HS_0;
my $max_del_size = 1000000;
my $min_HS_id=97;
my $min_LS_id=80;
my $number_high_stringency_clusters=0;
my %number_high_stringency;
my $maximum_good_blat_results_needed=2;
my $deletion_size_tolerance = 0.3;
my $exclude_random_chromosomes=1;
my $ignore_complex_events = 0; #Jan on July 3rd 2008
my $singleton_complex_insertion_size_maximum = 100000;
my $min_in_cluster = 2; # Alex

unless (@ARGV) {
	print STDERR "params: infile [transloc;default=OFF] [also_report_HS_0;default=OFF] [blat_comparison_perc_id_distance(integer);default=OFF] [combined_linkermapped_file.fa;needed_for_blat] [min_concordant;default=$min_concordant","bp] [max_concordant;default=$max_concordant","bp] [mean_concordant;default=$mean_concordant] [min_in_cluster;default=$min_in_cluster] [ignore_complex_events;default=OFF] [blat_organism;e.g.:mouse/arabidopsis;default=human] [include_if_more_than_one_best_placement] [output_identical_and_near-identical_reads] [no_fuzzy_single_linkage_clustering] [HS_sim;default=$min_HS_id] [LS_sim;default=$min_LS_id] deletion_size_tolerance[default=$deletion_size_tolerance] complex_insertion_size_maximum[default=$singleton_complex_insertion_size_maximum]   max_del_size[default=$max_del_size]\n";
	print STDERR " --> exlcude random is now on!\n";
	print STDERR "--> to save time, blat is performed only after linkage clustering!\n";
	print STDERR "        --> including more than one best placement also not fully implemented yet!\n";
	print STDERR "        --> procedure: find overlaps can speed up by factor of two, if sorting of start coordinates implemented\n";
	print STDERR "        ---> near-identical pairs: one read has same start and end --> amplification?\n";
	print STDERR "\n\tcriteria:\n\t1(DEL). overlap and same orientations\n";
	print STDERR "\t2(DEL). predicted deletion sizes are consistent\n";
	print STDERR "\t3(DEL). smallest possible deletion fits between smallest_end_coordinate-biggest_start_coordinate (check: what are the error bars here?)   - clustering non-perfect, but OK for now: may wanna check with Nick Carriero!\n";	
	print STDERR "\n\t1(INS). overlap and same orientations\n";#currently nothing more!;
	print STDERR "\n\t2(INS). there must be space of at least \'0\' for the insertion, thus biggest startB<smallest endB!\n";
	print STDERR "\n\n\t1(INV). overlap, and inverted orientations\n";#currently nothing more!;
	print STDERR "\n\t2(INV). left and right distances of starts/ends must together be <= 1*discrepancy!\n"; #XXX
	print STDERR "\n\t3(INS). there must be space of at least \'0\' for the insertion, thus biggest startB<smallest endB!\n";#XXX
	print STDERR "\n       --> Celera genome is automatically recognized\n";
	print STDERR "\n\n     --> now activated deletion_size_tolerance!!\n";
	exit;
}

#-- main
my ($infile,$do_transloc,$report_HS_0,$blat_comparison_perc_id_distance,$combined_linkermapped_file,$min,$max,$mean,$NEWmin_in_cluster,$NEW_ignore_complex_events, $blat_organism,$include_if_more_than_one_best_placement, $output_identical_and_near_identical_reads, $no_linkage_clustering, $new_HS, $new_LS, $newdeletion_size_tolerance, $newsingleton_complex_insertion_size_maximum, $new_max_del_size) = @ARGV;
$deletion_size_tolerance = $newdeletion_size_tolerance if (defined($newdeletion_size_tolerance));
if (defined($min)) {
	$min_concordant = $min;
}
if ($NEWmin_in_cluster) {
	$min_in_cluster = $NEWmin_in_cluster;
}
if ($NEW_ignore_complex_events) {
	$ignore_complex_events = $NEW_ignore_complex_events;
}
if ($blat_comparison_perc_id_distance) {
   if ($blat_comparison_perc_id_distance =~ /\./) {
	print STDERR "blat_comparison_perc_id_distance must be an integer!\n";
	exit;
   }
}
if (defined ($blat_organism)) {
	if ($blat_organism eq "arabidopsis") {
		print STDERR "arabidopsis blat path missing!\n";die;
	} elsif ($blat_organism eq "mouse") {
		$genome_DB = "/home2/jk377/Data/Genomes/UCSC/Mouse_build_mm9/mm9.2bit";
	} elsif ($blat_organism eq "0") {
		$blat_organism = undef;	
	}  elsif ($blat_organism ne "human") {
		print STDERR "$blat_organism blat path missing!\n";die;
	}
	print STDERR "running blat in $blat_organism: $genome_DB ...\n" if ($blat_organism);
} else {
	$blat_organism = "human"; #will be ignored if no linker-file given
}
if (defined($newsingleton_complex_insertion_size_maximum)) {
	$singleton_complex_insertion_size_maximum = $newsingleton_complex_insertion_size_maximum;
}
if ($do_transloc) {
	$translocations = 1;
}
if (defined($max)) {
	$max_concordant = $max;
}
if ($new_max_del_size) {
	$max_del_size = $new_max_del_size;
	print STDERR "option: max_del_size=$max_del_size\n";
}
$also_report_HS_0 = $report_HS_0 if ($report_HS_0);
if (defined($mean)) {
	$mean_concordant = $mean;
}
$min_HS_id = $new_HS if ($new_HS);
$min_LS_id = $new_LS if ($new_LS);

my $max_discrepancy = $max_concordant-$min_concordant+1;
my (%CHR, %NON_TRANSL_CALLs, %TRANSL_CALLs, %number_of_calls, %IDs, %STRAND_As, %STARTs, %ENDs,%STARTbs, %ENDbs, @TRANSL_CALLs, %PREDICTED_EVENT_SIZE, @SLclust, %SLclust_hash, %SEQ_IDs);
my (%T_IDs, %T_STARTs_B, %T_ENDs_B, %T_STARTs_A, %T_ENDs_A, %T_ASSIGN, %TRANSLOC_SEEN);
my (%highest_score, %posbest_with_distance, %best_with_distance, %col_lab, %HighStringency);
&parse_blat_lookup_table if ($blat_comparison_perc_id_distance);
&parse_infile ($infile);

#print STDERR "disabling Del, InsSim, Coom, Inv for faster search!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
print STDERR "# complex insertion max-size = $singleton_complex_insertion_size_maximum bp #\n";
my $numb_del_clusters = &find_overlaps_deletions;
&run_blat ($numb_del_clusters) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_del_clusters);
&output_clusters ($numb_del_clusters, 'Deletion') unless ($no_linkage_clustering);
my $numb_ins_clusters = &find_overlaps_insertions;
&run_blat ($numb_ins_clusters) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_ins_clusters);
&output_clusters ($numb_ins_clusters, 'Insertion') unless ($no_linkage_clustering);
my $numb_inv_clusters_A = &find_overlaps_inversions_A;
&run_blat ($numb_inv_clusters_A) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance  and $numb_inv_clusters_A);
&output_clusters ($numb_inv_clusters_A, 'Inversion(-L+)') unless ($no_linkage_clustering);
my $numb_inv_clusters_B = &find_overlaps_inversions_B;
&run_blat ($numb_inv_clusters_B) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_inv_clusters_B);
&output_clusters ($numb_inv_clusters_B, 'Inversion(+L-)') unless ($no_linkage_clustering);
my $numb_compl_clusters = &find_overlaps_complex;
&run_blat ($numb_compl_clusters) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_compl_clusters);
&output_clusters ($numb_compl_clusters, 'Complex') unless ($no_linkage_clustering);
my $numb_complex_insertions_1 = &find_overlaps_putative_complex_insertions('1');
&run_blat ($numb_complex_insertions_1) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_complex_insertions_1);
&output_clusters ($numb_complex_insertions_1, 'Insertion/Complex_T1') unless ($no_linkage_clustering);

my $numb_complex_insertions_2 = &find_overlaps_putative_complex_insertions('2');
&run_blat ($numb_complex_insertions_2) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance and $numb_complex_insertions_2);
&output_clusters ($numb_complex_insertions_2, 'Insertion/Complex-T2') unless ($no_linkage_clustering);

#if ($translocations)  {
#	my $numb_transl_clusters = &find_overlaps_translocations;
#	#&run_blat ($numb_transl_clusters) if (!$no_linkage_clustering and $blat_comparison_perc_id_distance);
#	&output_clusters ($numb_ins_clusters, 'Translocation-as-yet-without-blat') unless ($no_linkage_clustering);
#}
#Inversion(-L+)
#print STDERR "Final clusters (LS): DEL=",$number_high_stringency{"Deletion"},"; INS=",$number_high_stringency{"Insertion"},"; INV=",$number_high_stringency{"Inversion(+L-)"}+$number_high_stringency{"Inversion(-L+)"},"; COMPL=",$number_high_stringency{"Complex"},", complex-insertions=",$number_high_stringency{'Insertion/Complex'}," (total number of high-stringency-clusters=$number_high_stringency_clusters);\n" unless $no_linkage_clustering;
#system "rm .tmp_blat$$"; #removed earlier
exit;

##################
#-- subs   #######
##################

sub parse_infile {
	#print "CIRCLE_ID\tPLACEMENT_SCORE\tCHR_A\tSTART_A\tEND_A\tSTRAND_A\tPERC_SEQID_UNWEIGHED_A\tLENGTH_A\tCHR_B\tSTART_B\tEND_B\tSTRAND_B\tPERC_SEQID_UNWEIGHED_B\tLENGTH_B\tCALL\n";
	my $file = $_[0];
	if ($file =~ /\.gz$/)  {
		open FH, "gunzip -c $file |" or die;
	} else {
		open FH, $file or die;
	}
	print STDERR "Parsing $file ... ";
	%col_lab = ();
    	my $first_line = <FH>;
    	chomp $first_line;
	my (@col) = split (/[\t ]+/, $first_line);
	for (my $i=0; $i<@col; $i++) {
	      $col_lab{$col[$i]}=$i;
	}
	my $cnt=0;
	my $del=0;my $ins=0; my $inv=0;my $compl=0;
	my %seen;
	while (<FH>) {
		chomp;
		next unless ($_);
		next if ($seen{$_});
		$seen{$_}=1;
		next if ($_ =~ /^\#/);
		next if ($_ =~ /^CIRCLE_ID/);
		my (@col) = split (/[\t ]+/);
		my $CIRCLE_ID = $col[$col_lab{"CIRCLE_ID"}];
		my $CHR_A = $col[$col_lab{"CHR_A"}];
		my $CHR_B = $col[$col_lab{"CHR_B"}];
		my $startA=$col[$col_lab{"START_A"}];
		my $endA=$col[$col_lab{"END_A"}];
		my $startB =$col[$col_lab{"START_B"}];
		my $endB=$col[$col_lab{"END_B"}];
		my $seqidA=$col[$col_lab{"PERC_SEQID_UNWEIGHED_A"}];
		my $seqidB=$col[$col_lab{"PERC_SEQID_UNWEIGHED_B"}];
		my ($lowest_ID) = sort {$a<=>$b} ($seqidA,$seqidB);
		next if ($lowest_ID < $min_LS_id);
		#print STDERR "$lowest_ID $seqidA $seqidB\n";
		$number_of_calls{$CIRCLE_ID}++;
		if ($CHR_A eq $CHR_B) {
			$CELERA=1 if ($CHR_A =~ /chromosome/);
			#-- fix for celera
			(undef, $CHR_A) = split (/chr/, $CHR_A);
			$CHR_A = "chr$CHR_A";
			$CHR_A = "celera.$CHR_A" if ($CHR_A=~ /chromosome/);
			$CHR_A =~ s/celera\.$//;
			(undef, $CHR_B) = split (/chr/, $CHR_B);
			$CHR_B = "chr$CHR_B";
			$CHR_B = "celera.$CHR_B" if ($CHR_B=~ /chromosome/);
			$CHR_B =~ s/celera\.$//;						
			my $complex=0;
			$NON_TRANSL_CALLs{$CIRCLE_ID} = $_;
			my (@sorted) = sort {$a<=>$b} ($col[$col_lab{"START_A"}],$col[$col_lab{"END_A"}],$col[$col_lab{"START_B"}],$col[$col_lab{"END_B"}]);
			#print STDERR ">>$sorted[0] $sorted[-1]\n" if ($CIRCLE_ID eq 'ECL5JZE02FENUT');
			#--- first discard overlapping placements, pairs should never overlap!
			if ($col[$col_lab{"STRAND_A"}] eq $col[$col_lab{"STRAND_B"}]) { #non-inversion
				if (($startA >= $startB and $startA <= $endB) or ($startB>=$startA and $startB<=$endA)) {
					$complex=1; #case A--> embedded reads!
					#print STDERR "embedded_reads_discard $_\n";#XXX
				} elsif (($col[$col_lab{"STRAND_A"}] eq '-' and $col[$col_lab{"STRAND_B"}] eq '-' and $startA < $startB) 
				  or ($col[$col_lab{"STRAND_A"}] eq '+' and $col[$col_lab{"STRAND_B"}] eq '+' and $startA > $startB))	{
					#print STDERR "nonembedded_reads $_\n";#XXX
					if (($sorted[-1] - $sorted[0]) >= $max_concordant) {
						if ($ignore_MT_DNA_origin and ($CHR_A =~ /chrM/)) {
							my ($low, $high) = sort {$a<=>$b} ($startA, $startB);
							next if (($high > 16500-($max_concordant-$min_concordant)) and ($low < ($max_concordant-$min_concordant)));
						}
						next if ((abs($sorted[3]-$sorted[0])-10000)>$max_del_size);
						push (@{$IDs{$CHR_A}[0]},$CIRCLE_ID);
						push (@{$STARTs{$CHR_A}[0]},$sorted[0]);
						push (@{$STARTbs{$CHR_A}[0]},$sorted[1]);
						push (@{$ENDbs{$CHR_A}[0]},$sorted[2]);
						push (@{$ENDs{$CHR_A}[0]},$sorted[3]);
						push (@{$PREDICTED_EVENT_SIZE{$CHR_A}[0]}, ($sorted[-1]-$sorted[0]-$mean_concordant));
						push (@{$SEQ_IDs{$CHR_A}[0]},$lowest_ID);
						$del++;
					} elsif (($sorted[-1] - $sorted[0]) <= $min_concordant) {
						push (@{$IDs{$CHR_A}[1]},$CIRCLE_ID);
						push (@{$STARTs{$CHR_A}[1]},$sorted[0]);
						push (@{$STARTbs{$CHR_A}[1]},$sorted[1]);
						push (@{$ENDbs{$CHR_A}[1]},$sorted[2]);
						push (@{$ENDs{$CHR_A}[1]},$sorted[3]);
						push (@{$PREDICTED_EVENT_SIZE{$CHR_A}[1]}, ($mean_concordant-($sorted[-1]-$sorted[0])));
						push (@{$SEQ_IDs{$CHR_A}[1]},$lowest_ID);
						$ins++;
					} else {
						print STDERR "warning; found concordant event in call file!?? ignoring $CIRCLE_ID\n";
						next;
					}
				} else {
					$complex=1; #case B
				}
				if ($complex) {
					next if ($ignore_complex_events); #Jan on July 3rd 2008
					if ($ignore_MT_DNA_origin and ($CHR_A =~ /chrM/)) {
                          my ($low, $high) = sort {$a<=>$b} ($startA, $startB);
                          next if (($high > 16500-($max_concordant-$min_concordant)) and ($low < ($max_concordant-$min_concordant)));
					}																				
					push (@{$IDs{$CHR_A}[3]},$CIRCLE_ID);
					push (@{$STARTs{$CHR_A}[3]},$sorted[0]);
                    push (@{$STARTbs{$CHR_A}[3]},$sorted[1]);
                    push (@{$ENDbs{$CHR_A}[3]},$sorted[2]);
                    push (@{$ENDs{$CHR_A}[3]},$sorted[3]);
                    push (@{$PREDICTED_EVENT_SIZE{$CHR_A}[3]}, ($sorted[-1]-$sorted[0])); #approximation for complex cases
                    push (@{$SEQ_IDs{$CHR_A}[3]},$lowest_ID);
					$compl++;
				}
			} else {
				push (@{$IDs{$CHR_A}[2]},$CIRCLE_ID);
				push (@{$STRAND_As{$CHR_A}[2]},$col[$col_lab{"STRAND_A"}]);
				#print STDERR "$STRAND_As{$CHR_A}[2][-1]\n";
				push (@{$STARTs{$CHR_A}[2]},$sorted[0]);
				push (@{$STARTbs{$CHR_A}[2]},$sorted[1]);
				push (@{$ENDbs{$CHR_A}[2]},$sorted[2]);
				push (@{$ENDs{$CHR_A}[2]},$sorted[3]);
				push (@{$PREDICTED_EVENT_SIZE{$CHR_A}[2]}, "NaN");
				push (@{$SEQ_IDs{$CHR_A}[2]},$lowest_ID);
				$inv++;
			}
		} else {
			next unless ($translocations);
			my ($CHR_1, $CHR_2) = sort ($CHR_A, $CHR_B);
			push (@TRANSL_CALLs, @col);
			$TRANSLOC_SEEN{"$CHR_1.$CHR_2"}=1;
			push (@{$T_IDs{"$CHR_1.$CHR_2"}}, $CIRCLE_ID);
			push (@{$T_STARTs_A{"$CHR_1.$CHR_2"}}, $startA);
			push (@{$T_STARTs_B{"$CHR_1.$CHR_2"}}, $startB);
			push (@{$T_ENDs_A{"$CHR_1.$CHR_2"}}, $endA);
			push (@{$T_ENDs_B{"$CHR_1.$CHR_2"}}, $endB);
			push (@{$SEQ_IDs{"$CHR_1.$CHR_2"}},$lowest_ID);
			if ($CHR_A eq $CHR_1) {
				push (@{$T_ASSIGN{"$CHR_1.$CHR_2"}}, 0);
			} else {
				push (@{$T_ASSIGN{"$CHR_1.$CHR_2"}}, 1);
			}
			#print STDERR "$CHR_1}{$CHR_2\n";
		}
		$CHR{$CHR_A}=1;
		$CHR{$CHR_B}=1;
		$cnt++;
	}
	print STDERR "$cnt lines parsed; ",scalar (keys %CHR)," chromsosome(s); putative_del=$del; putative_ins=$ins; putative_inv=$inv; putative_complex=$compl.\n";
	close FH;
}


sub find_overlaps_deletions {
	my $cnt=0;
	my $overl=0;
	my $ident=0;
	my $cluster_id=0;
	my %seen_comb;
	#%HighStringency=();
	print STDERR "Looking for overlaps (agreeing deletions)... \n";
	foreach my $chr (sort keys %CHR) {
		#print STDERR "$chr ";
		next unless ($STARTs{$chr}[0][1]);
		for (my $i=0; $i<@{$STARTs{$chr}[0]}-1; $i++) {
		#print STDERR "$IDs{$chr}[0][$i]...\n";
			next if ($number_of_calls{$IDs{$chr}[0][$i]}>1); #ignored from very beginning!!! OK
			for (my $y=$i+1; $y<@{$STARTs{$chr}[0]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[0][$y]}>1);
				#1. see whether predicted sizes agree
				if (abs($PREDICTED_EVENT_SIZE{$chr}[0][$i]-$PREDICTED_EVENT_SIZE{$chr}[0][$y]) <= $max_discrepancy) {
					#print STDERR "found:", abs($PREDICTED_EVENT_SIZE{$chr}[0][$i]-$PREDICTED_EVENT_SIZE{$chr}[0][$y]), "\n";
					#2. -- assess overlap
					if (($STARTs{$chr}[0][$i] >= $STARTs{$chr}[0][$y] && $STARTs{$chr}[0][$i] <= $ENDs{$chr}[0][$y]) or ($STARTs{$chr}[0][$i]<=$STARTs{$chr}[0][$y] && $STARTs{$chr}[0][$y]<=$ENDs{$chr}[0][$i])) { 
						#3. --- does deletion fit in there? 
						my ($smaller_event_size) = sort {$a<=>$b} ($PREDICTED_EVENT_SIZE{$chr}[0][$i],$PREDICTED_EVENT_SIZE{$chr}[0][$y]);		
						my ($biggest_start_b) = sort {$b<=>$a} ($STARTbs{$chr}[0][$i],$STARTbs{$chr}[0][$y]);
						my ($smallest_end_b) = sort {$a<=>$b} ($ENDbs{$chr}[0][$i],$ENDbs{$chr}[0][$y]);
						##if ((($smallest_end_b-$biggest_start_b-1) > $smaller_event_size-$max_discrepancy) && ($smallest_end_b-$biggest_start_b-1) >= ($max_concordant - $mean_concordant)) {#3. find correct expression for max_discrepancy #XXX --> exclude overlaps!!
						if ((($smallest_end_b-$biggest_start_b-1) > $smaller_event_size-$max_discrepancy) && ($smallest_end_b-$biggest_start_b-1) >= ($max_concordant - $mean_concordant) * (1-$deletion_size_tolerance)) { #included deletion_size_tolerance [currently inactivated] XXX
						#OLD until May 6th: if ( abs($smallest_end_b-$biggest_start_b-1) > $smaller_event_size-$max_discrepancy) {#3. find correct expression for max_discrepancy #XXX --> exclude overlaps!!
							if (!$STARTs{$chr}[0][$i] or !$STARTs{$chr}[0][$y] or !$ENDs{$chr}[0][$i] or !$ENDs{$chr}[0][$y]) {
								print STDERR "UNINITIALIZED: !$STARTs{$chr}[0][$i] or !$STARTs{$chr}[0][$y] or !$ENDs{$chr}[0][$i] or !$ENDs{$chr}[0][$y] ($i/$y)\n";
								die;
							}
							if (($STARTs{$chr}[0][$i] == $STARTs{$chr}[0][$y] and $ENDs{$chr}[0][$i] == $ENDs{$chr}[0][$y]) or ($STARTbs{$chr}[0][$i] == $STARTbs{$chr}[0][$y] and $ENDbs{$chr}[0][$i] == $ENDbs{$chr}[0][$y]) or ($STARTs{$chr}[0][$i] == $STARTs{$chr}[0][$y] and $STARTbs{$chr}[0][$i] == $STARTbs{$chr}[0][$y]) or ($ENDs{$chr}[0][$i] == $ENDs{$chr}[0][$y] and $ENDbs{$chr}[0][$i] == $ENDbs{$chr}[0][$y])) {
								print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$y]} ($i/$y $STARTs{$chr}[0][$i]/$STARTs{$chr}[0][$y] $ENDs{$chr}[0][$i]/$ENDs{$chr}[0][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[0][$i]/$PREDICTED_EVENT_SIZE{$chr}[0][$y])\n\n" if ($output_identical_and_near_identical_reads); 
								$ident++;
							} else {
								print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$y]} ($i/$y $STARTs{$chr}[0][$i]/$STARTs{$chr}[0][$y] $ENDs{$chr}[0][$i]/$ENDs{$chr}[0][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[0][$i]/$PREDICTED_EVENT_SIZE{$chr}[0][$y])\n\n" if ($no_linkage_clustering); 
								#-- sort into cluster
								my $match = 0;
								foreach my $cluster_id (@{$SLclust[0]}) {
									if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]})) {
										$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$y]}=1;
										$match=1;
										#next;
									} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[0][$y]})) {
										$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]}=1;
										$match=1;
										#next;
									}
								}
								if (!$match) {
									$cluster_id++;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$y]}=1;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]}=1;
									push (@{$SLclust[0]}, $cluster_id);
									
								}
								#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[0][$i] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[0][$i].$cluster_id"}));
								#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[0][$y] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[0][$y].$cluster_id"}));
								#$seen_comb{"$SEQ_IDs{$chr}[0][$y].$cluster_id"}=1;
								#$seen_comb{"$SEQ_IDs{$chr}[0][$i].$cluster_id"}=1;
							}
							#print ">$smaller_event_size | $biggest_start_b | $smallest_end_b\n";
							$overl++;
						}						
					}
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Losses: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return $cluster_id;
}


sub find_overlaps_putative_complex_insertions {
    my ($type) = @_;
	my $cnt=0;
    my $overl=0;
    my $ident=0;
    my $cluster_id=0;
    my %seen_comb;
    print STDERR "Looking for overlaps (agreeing complex $type; max_discrepancy=$max_discrepancy)... \n";
    foreach my $chr (sort keys %CHR) {
		next unless ($STARTs{$chr}[0][0]); #deletion-like
		next unless ($STARTs{$chr}[3][0]); #complex-like
		for (my $i=0; $i<@{$STARTs{$chr}[0]}; $i++) {
			#print STDERR "$IDs{$chr}[0][$i]...\n";
			next if ($number_of_calls{$IDs{$chr}[0][$i]}>1); #ignored from very beginning!!! OK
			for (my $y=0; $y<@{$STARTs{$chr}[3]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[3][$y]}>1);
			    #1. -- initial size assessment 	
				if (((abs($STARTs{$chr}[0][$i] - $STARTs{$chr}[3][$y])<=$max_discrepancy) and ($type == 1)) or ((abs($ENDs{$chr}[0][$i] - $ENDs{$chr}[3][$y])<=$max_discrepancy) and ($type == 2))) {# smaller one will be at predicted insertion site 
					#print STDERR "$STARTs{$chr}[0][$i]  $STARTs{$chr}[3][$y] 
					#2. -- assess overlap
					if (($STARTs{$chr}[0][$i] < $STARTs{$chr}[3][$y] && $STARTs{$chr}[3][$y] < $ENDs{$chr}[0][$i]) or ($STARTs{$chr}[3][$y] < $STARTs{$chr}[0][$i] && $STARTs{$chr}[0][$i] < $ENDs{$chr}[3][$y])) {
						#3. -- see whether predicted sizes agree (should not be too large)
						if ((abs($ENDs{$chr}[0][$i]-$ENDs{$chr}[3][$y]) <= $singleton_complex_insertion_size_maximum ) and (abs($STARTs{$chr}[0][$i]-$STARTs{$chr}[3][$y]) <= $singleton_complex_insertion_size_maximum )){
						#print STDERR "found:", abs($PREDICTED_EVENT_SIZE{$chr}[0][$i]-$PREDICTED_EVENT_SIZE{$chr}[0][$y]), "\n";
							if (!$STARTs{$chr}[0][$i] or !$STARTs{$chr}[3][$y] or !$ENDs{$chr}[0][$i] or !$ENDs{$chr}[3][$y]) {
								print STDERR "UNINITIALIZED: !$STARTs{$chr}[0][$i] or !$STARTs{$chr}[3][$y] or !$ENDs{$chr}[0][$i] or !$ENDs{$chr}[3][$y] ($i/$y)\n";
								die;
							}
							if (0) { #identity as below cannot happen here!
							#if (($STARTs{$chr}[0][$i] == $STARTs{$chr}[3][$y] and $ENDs{$chr}[0][$i] == $ENDs{$chr}[0][$y]) or ($STARTbs{$chr}[0][$i] == $STARTbs{$chr}[0][$y] and $ENDbs{$chr}[0][$i] == $ENDbs{$chr}[0][$y]) or ($STARTs{$chr}[0][$i] == $STARTs{$chr}[0][$y] and $STARTbs{$chr}[0][$i] == $STARTbs{$chr}[0][$y]) or ($ENDs{$chr}[0][$i] == $ENDs{$chr}[0][$y] and $ENDbs{$chr}[0][$i] == $ENDbs{$chr}[0][$y])) {
								print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$y]} ($i/$y $STARTs{$chr}[0][$i]/$STARTs{$chr}[0][$y] $ENDs{$chr}[0][$i]/$ENDs{$chr}[0][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[0][$i]/$PREDICTED_EVENT_SIZE{$chr}[0][$y])\n\n" if ($output_identical_and_near_identical_reads);
								$ident++;
							} else {
								print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[0][$y]} ($i/$y $STARTs{$chr}[0][$i]/$STARTs{$chr}[0][$y] $ENDs{$chr}[0][$i]/$ENDs{$chr}[0][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[0][$i]/$PREDICTED_EVENT_SIZE{$chr}[0][$y])\n\n" if ($no_linkage_clustering);
								#-- sort into cluster
								my $match = 0;
								foreach my $cluster_id (@{$SLclust[4]}) {
									if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]}=1;
									$match=1;
									#next;
									} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]})) {
										$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]}=1;
										$match=1;
										#next;
									}
								}
								if (!$match) {
									$cluster_id++;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]}=1;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[0][$i]}=1;
									push (@{$SLclust[4]}, $cluster_id);
								}
							}
							$overl++;
						}
					}
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Putative complex insertions: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return $cluster_id;
}

sub run_blat {
	my $number_of_clusters = $_[0];
	%highest_score=();
	%posbest_with_distance=();
	%best_with_distance=(); 
	my %seen;
	my $cnt=0;
	if ($CELERA) {
		$genome_DB = "~/Data/Genomes/Celera_Human/celera_human_all_sequences.2bit";
		$ooc_file = "~/Data/Genomes/Celera_Human/celera_human_all_sequences.11.ooc";
	}
	for  (my $i=1; $i<=$number_of_clusters; $i++) {
		foreach my $id (sort keys %{$SLclust_hash{$i}}) {
			$seen{$id}=1;
			$cnt++;
		}
	}
	#print STDERR "processing $cnt predicted deletions.";
	my ($tmpreadfile, $number_reads_2_blat) = &parse_and_write_reads (keys %seen);
	if ($number_reads_2_blat) {
		my $min_blat_seq_id_to_consider = $blat_seq_ID_cutoff-$blat_comparison_perc_id_distance;
		print STDERR "now running blat with min_blat_seq_id_to_consider=$min_blat_seq_id_to_consider (required_distance=$blat_comparison_perc_id_distance) on $tmpreadfile ($number_reads_2_blat reads)...\n";
		my $minScore=$min_read_match_length-$blat_comparison_perc_id_distance;
		#unless (glob "11.ooc*") {
		#	print STDERR "file ./11.ooc is missing!\n";
		#	die;
		#}
		#my $blatrun = "~/bin/blat/blat-x86_64-suse-linux $genome_DB $tmpreadfile -noHead -noTrimA -minIdentity=$min_blat_seq_id_to_consider -minScore=$minScore -tileSize=11 -ooc=$ooc_file $tmpreadfile.psl"; #noTrimA does not trim polyA; -fastMap ignores introns = fast DNA/DNA remapping -minScore=N sets minimum score.  This is the matches minus the mismatches minus some sort of gap penalty; noHead=no_outfile_header
		my $blatrun = "~/bin/blat/blat-x86_64-suse-linux $genome_DB $tmpreadfile -noHead -noTrimA -minIdentity=$min_blat_seq_id_to_consider -minScore=$minScore -tileSize=11  $tmpreadfile.psl"; #noTrimA does not trim polyA; -fastMap ignores introns = fast DNA/DNA remapping -minScore=N sets minimum score.  This is the matches minus the mismatches minus some sort of gap penalty; noHead=no_outfile_header
		
		print STDERR "$blatrun\n";
		system "$blatrun";
		open FHBL, "$tmpreadfile.psl" or die;
		while (<FHBL>) {
			chomp;
			my (@col) = split;
			#print "match=$col[0]; mismatch=$col[1]; gapQ=$col[5]; gapT=$col[7]; id=$col[9]; chr=$col[13]; start=$col[15]; end=$col[15]\n"; 
			my ($match, $mismatch,$gapQ,$gapT,$id,$chr,$start,$end) = ($col[0],$col[1],$col[5],$col[7],$col[9],$col[13],$col[15]+1,$col[16]); #ssem to be a blat bug that you have to add '1' on the left!
			#print "> match=$match; mismatch=$mismatch; gapQ=$gapQ; gapT=$gapT; id=$id; chr=$chr; start=$start; end=$end\n";
			my $new_chr=$chr;
			#print STDERR "$chr\n";
			if ($exclude_random_chromosomes) {
				next if ($chr =~ /_/);
			}
			if ($chr=~/(chromosome [0-9A-Za-z_]+)/) {
   	           $new_chr = $1;
		          if ($new_chr =~ /CRA_TCAGchr7v2/) {
				     $new_chr.= "_CRA_TCAGchr7v2";
				  }
			} 
		    $new_chr=~s/ //;
			$chr = $new_chr;
			#print STDERR "->$chr\n";
			my $ad_hoc_score = $match - 0.8 * ($mismatch+$gapT+$gapQ);
			if (!exists($highest_score{$id})) {
				$best_with_distance{$id} = $ad_hoc_score;
				$posbest_with_distance{$id} = "$chr:$start-$end";
				$highest_score{$id} = $ad_hoc_score;
			} elsif (($ad_hoc_score - $blat_comparison_perc_id_distance) >= $highest_score{$id}) {
				$best_with_distance{$id} = $ad_hoc_score;
				$posbest_with_distance{$id} = "$chr:$start-$end";
				$highest_score{$id} = $ad_hoc_score;
			} elsif (($ad_hoc_score + $blat_comparison_perc_id_distance) <= $highest_score{$id}) {
				#-- keep everything as it is
			} else {
				#-- too close
				delete ($best_with_distance{$id});
				delete ($posbest_with_distance{$id});
				$highest_score{$id} = $ad_hoc_score if ($ad_hoc_score>$highest_score{$id}); #needs to be kept and updated
			}
		}
		close FHBL;
		system "rm -rf $tmpreadfile.psl";
	} #end if reads to run blat for
    system "rm -rf $tmpreadfile";
	print STDERR "Best scores with distance: $blat_comparison_perc_id_distance\n";
	#foreach my $id (sort keys %highest_score) {
		#print "$id; highest_score=$highest_score{$id}; ";
	#	if (exists($best_with_distance{$id})) {
			#print "$best_with_distance{$id}-> $posbest_with_distance{$id}";
	#	}
		#print "\n";
	#}
	#print STDERR "need to determine match between best and thought best coordiantes!!\n";
	#die;
}

sub parse_and_write_reads {
	my (%id, %seq);
	foreach (@_) { #get IDs
		$id{$_}=1;
		#print STDERR ">>$_:$id{$_}\n";
	}
	#print STDERR "preparing blat to test these ",scalar (keys %id)," cases: ";
	if (!defined($combined_linkermapped_file)) {
		print STDERR "ERROR: \"combined_linkermapped_file.fa\" undefined!\n";
		die;
	}
	my $file = $combined_linkermapped_file;
	if ($file =~ /\.gz$/) {
		open FH, "gunzip -c $file |" or die;
	} else {
		open FH, "$file" or die;
	}
	print STDERR "Parsing $file ... ";
	my $match;
	my $reads_generated=0;
	my $cnt=0;
	#-- get reads
	while (<FH>) {
		chomp;
		if ($_ =~ /^>([0-9a-zA-Z\.\-\_]+)/) {
			my $test_id=$1;
			(undef, $test_id) = split (/\./, $test_id);
			$match = $test_id;
			$test_id =~ s/_[AB]$//;
		#	print STDERR "check 1:$test_id:$match\n";
			#print STDERR "check 2:$test_id:$match:$id{$test_id}:",join (" ", keys %id),"\n";
			if (!exists($id{$test_id})) { #ignore!
				$match = "";
				$cnt--;
			}
		        $cnt++;	
		} elsif ($match) {
			$seq{$match}=$_;
		}
	}
	print STDERR "", scalar (keys %seq) ," reads. \n";
	#-- print reads
	open WHBLAT, "> .tmp_blat$$" or die;
	foreach my $read_id (sort keys %seq) {
		my $circle = $read_id;
		$circle =~ s/_.//;
		if ($PREVIOUSLY_EVALUATED{$circle}) {
			#print STDERR "HALLO: ignoring $read_id; looked at before!\n";
			next if ($PREVIOUSLY_EVALUATED{$circle}>1); #two reads needed
		}
		print WHBLAT ">$read_id\n$seq{$read_id}\n";
		$PREVIOUSLY_EVALUATED{$circle}++;
		$reads_generated++;
	} 
	close WHBLAT;
	return (".tmp_blat$$", $reads_generated);
}

sub output_clusters {
	my ($number_of_clusters, $label) = @_;
	for  (my $i=1; $i<=$number_of_clusters; $i++) {
		my $cnt=0;
		my $HS_score=0;
		foreach my $id (sort keys %{$SLclust_hash{$i}}) {
			my (@col) = split (/[ \t]+/, $NON_TRANSL_CALLs{$id});
			my $seqidA=$col[$col_lab{"PERC_SEQID_UNWEIGHED_A"}];
			my $seqidB=$col[$col_lab{"PERC_SEQID_UNWEIGHED_B"}];
			my ($lowest_ID) = sort {$a<=>$b} ($seqidA,$seqidB);
			$HS_score++ if ($lowest_ID >= $min_HS_id);
												
		}
		unless ($also_report_HS_0) {
			next unless ($HS_score);
		}
		#-- inserted by Jan on 16.6.08 (allows for clusters of size 3, 4, 5, etc.; avoids non-unique events due to library-redundnacy)
		#if (keys(%{$SLclust_hash{$i}}) < $min_in_cluster) { next; }
		my $cluster_cnt_unique=0;
		my (@startA,@endA,@startB,@endB);
		foreach my $key (sort keys(%{$SLclust_hash{$i}})) {
			#print STDERR "hallo:$NON_TRANSL_CALLs{$key}\n";
			my (undef, undef, undef, $stA, $eA, undef, undef, undef, undef, $stB, $eB) = split (/[ \t]/, $NON_TRANSL_CALLs{$key});
			($stA,$eA,$stB,$eB) = sort {$a<=>$b} ($stA,$eA,$stB,$eB);
			if (@startA) {
				my $redund = 0;
				for (my $i=0; $i<@startA; $i++) {
					if ((abs($stA-$startA[$i]) <= 20) || (abs($eA-$endA[$i]) <= 20) || (abs($stB-$startB[$i]) <= 20) || (abs($eB-$endB[$i]) <= 20)) {
						$redund++ if (abs($stA-$startA[$i]) <= 20);
						$redund++ if (abs($eA-$endA[$i]) <= 20);
						$redund++ if (abs($stB-$startB[$i]) <= 20);
                                                $redund++ if (abs($eB-$endB[$i]) <= 20);
						last if ($redund>=3);
						$redund=0; #set to zero again!
					}
				}
				$cluster_cnt_unique++ unless ($redund>=3); #penalize if 3 out of 4 start/endpoints indicate redundancy; if begin/end within 20 bp in three cases, this is likely (20bp makes sure that non-perfect matches/read-errors are penalized as well!)
			} else {
				$cluster_cnt_unique++;
			}
			push (@startA, $stA); push (@endA, $eA); push (@startB, $stB); push (@endB, $eB);
			print STDERR "$stA, $eA, $stB, $eB - $cluster_cnt_unique...\n";			
		}
		next if ($cluster_cnt_unique < $min_in_cluster);
		#-- end inserted on 16.6.08 by Jan
 		$number_high_stringency_clusters++; #reordered by Jan on 16.6.08
		foreach my $id (sort keys %{$SLclust_hash{$i}}) {
			
			$overall_count++ unless ($cnt);
			my (@col) = split (/[ \t]+/, $NON_TRANSL_CALLs{$id});
			print "#\n#>$label-Cluster $overall_count (max-score=82;9-in-11 for INDELs; 0.73 for INVs; HS-Stringency-Score=$HS_score):\n" unless ($cnt);
			$number_high_stringency{$label}++ unless ($cnt);
			print "$NON_TRANSL_CALLs{$id}";
			my $readA="$id" . "_A";
			my $readB="$id" . "_B";
			my $matchcnt=0;
			if (exists($best_with_distance{$readA})) {
				#my @col = split (/[ \t]+/, $NON_TRANSL_CALLs{$id});
				$posbest_with_distance{$readA} =~ /(.*)\:([0-9]+)\-([0-9]+)/;
				my ($chr,$start,$end) = ($1,$2,$3); 
				#print STDERR "checking $readA ($NON_TRANSL_CALLs{$id}) with $chr,$start,$end\n";
				$matchcnt++ if ($start == $col[$col_lab{"START_A"}] and $end == $col[$col_lab{"END_A"}]);
				#print STDERR "checked against $col[$col_lab{START_A}]  $col[$col_lab{END_A}]\n";
			}
			if (exists($best_with_distance{$readB})) {
				#my @col = split (/[ \t]+/, $NON_TRANSL_CALLs{$id});
				$posbest_with_distance{$readB} =~ /(.*)\:([0-9]+)\-([0-9]+)/;
				my ($chr,$start,$end) = ($1,$2,$3);
				#print STDERR "checking $readB ($NON_TRANSL_CALLs{$id}) with $chr,$start,$end\n";
				$matchcnt++ if ($start == $col[$col_lab{"START_B"}] and $end == $col[$col_lab{"END_B"}]);
				#print STDERR "checked against $col[$col_lab{START_B}]  $col[$col_lab{END_B}]\n";
			}
			if ($matchcnt) {#XXX - changed on 2007-06-07
				print " [Distance_score($blat_comparison_perc_id_distance","bp)=$matchcnt]";				
				$PREVIOUSLY_SUCCESSFULLY_EVALUATED{$id}=$matchcnt;	
			} elsif ($PREVIOUSLY_SUCCESSFULLY_EVALUATED{$id}) {
				print " [Distance_score($blat_comparison_perc_id_distance","bp)=$PREVIOUSLY_SUCCESSFULLY_EVALUATED{$id}](calculated_previously)";
			}
			print "\n";
			$cnt++;
		}
	}
	#%{$SLclust_hash{$i}} = ();#make clean
	%SLclust_hash=();#make clean
	#print STDERR "size of hash after clean: ",scalar keys %SLclust_hash, " element 1: ", sort keys %{$SLclust_hash{1}}, "\n";
}
#sub output_insertion_clusters {#outdated!
#    my $number_of_clusters = $_[0];
#    for  (my $i=1; $i<=$number_of_clusters; $i++) {
#       	my $cnt=0;
#        foreach my $id (sort keys %{$SLclust_hash{$i}}) {
#		     print "#\n#>Insertion-Cluster $i (max_score=85;11-in-13):\n" unless ($cnt);
#			 print "$NON_TRANSL_CALLs{$id}\n";
#			 $cnt++;
#		}
#	}
#	#%{$SLclust_hash{$i}} = ();#make clean
#	%SLclust_hash=();#make clean
#	#print STDERR "size of hash after clean: ",scalar keys %SLclust_hash, " element 1: ", sort keys %{$SLclust_hash{1}}, "\n";
#}

#sub output_inversion_clusters {#outdated
#    my $number_of_clusters = $_[0];
#    for  (my $i=1; $i<=$number_of_clusters; $i++) {
#		my $cnt=0;
#		foreach my $id (sort keys %{$SLclust_hash{$i}}) {
#			print "#\n#>Inversion-Cluster $i (max_score=77;10-in-13):\n" unless ($cnt);
#		    print "$NON_TRANSL_CALLs{$id}\n";
#			$cnt++;
#		}
#	}
#	#%{$SLclust_hash{$i}} = ();#make clean
#	%SLclust_hash=();#make clean
#	#print STDERR "size of hash after clean: ",scalar keys %SLclust_hash, " element 1: ", sort keys %{$SLclust_hash{1}}, "\n";
#}


sub find_overlaps_insertions {
	my $cnt=0;
	my $overl=0;
	my $ident=0;
	my $cluster_id =0;
	my %seen_comb;
	#%HighStringency=();	
	print STDERR "Looking for overlaps (agreeing insertions)... \n";
	foreach my $chr (sort keys %CHR) {
		print STDERR "$chr ";
		next unless ($STARTs{$chr}[1][1]);
		for (my $i=0; $i<@{$STARTs{$chr}[1]}-1; $i++) {
		#print STDERR "$IDs{$chr}[1][$i]...\n";
			next if ($number_of_calls{$IDs{$chr}[1][$i]}>1);
			for (my $y=$i+1; $y<@{$STARTs{$chr}[1]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[1][$y]}>1);
				#1. -- assess overlap
				if (($STARTs{$chr}[1][$i] >= $STARTs{$chr}[1][$y] && $STARTs{$chr}[1][$i] <= $ENDs{$chr}[1][$y]) or ($STARTs{$chr}[1][$i]<=$STARTs{$chr}[1][$y] && $STARTs{$chr}[1][$y]<=$ENDs{$chr}[1][$i])) { 
					#.2. -- does insertion fit?
					my ($biggest_start_b) = sort {$b<=>$a} ($STARTbs{$chr}[1][$i],$STARTbs{$chr}[1][$y]);
					my ($smallest_end_b) = sort {$a<=>$b} ($ENDbs{$chr}[1][$i],$ENDbs{$chr}[1][$y]);
					#print "$IDs{$chr}[1][$i],$IDs{$chr}[1][$y],$biggest_start_b,$smallest_end_b \n" if ($IDs{$chr}[1][$i] eq "025722_3680_2097" or $IDs{$chr}[1][$y]  eq "025722_3680_2097");
					if ($biggest_start_b<$smallest_end_b) { 
					  #if ($smallest_end_b-$biggest_start_b) {#new as of May 6th
						if (($STARTs{$chr}[1][$i] == $STARTs{$chr}[1][$y] and $ENDs{$chr}[1][$i] == $ENDs{$chr}[1][$y]) or ($STARTbs{$chr}[1][$i] == $STARTbs{$chr}[1][$y] and $ENDbs{$chr}[1][$i] == $ENDbs{$chr}[1][$y]) or ($STARTs{$chr}[1][$i] == $STARTs{$chr}[1][$y] and $STARTbs{$chr}[1][$i] == $STARTbs{$chr}[1][$y]) or ($ENDs{$chr}[1][$i] == $ENDs{$chr}[1][$y] and $ENDbs{$chr}[1][$i] == $ENDbs{$chr}[1][$y])) {
						    print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[1][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[1][$y]} ($i/$y $STARTs{$chr}[1][$i]/$STARTs{$chr}[1][$y] $ENDs{$chr}[1][$i]/$ENDs{$chr}[1][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[1][$i]/$PREDICTED_EVENT_SIZE{$chr}[1][$y])\n\n"  if ($output_identical_and_near_identical_reads); 
							$ident++;
						} else {
							print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[1][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[1][$y]} ($i/$y $STARTs{$chr}[1][$i]/$STARTs{$chr}[1][$y] $ENDs{$chr}[1][$i]/$ENDs{$chr}[1][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[1][$i]/$PREDICTED_EVENT_SIZE{$chr}[1][$y])\n\n" if ($no_linkage_clustering); 
							#-- sort into cluster
							my $match = 0;
							foreach my $cluster_id (@{$SLclust[1]}) {
								if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[1][$i]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[1][$y]}=1;
									$match=1;
									#next;
								} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[1][$y]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[1][$i]}=1;
									$match=1;
									#next;
								}
							}
							if (!$match) {
								$cluster_id++;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[1][$y]}=1;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[1][$i]}=1;
								push (@{$SLclust[1]}, $cluster_id);
							}
							#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[1][$i] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[1][$i].$cluster_id"}));
							#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[1][$y] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[1][$y].$cluster_id"}));
							#$seen_comb{"$SEQ_IDs{$chr}[1][$y].$cluster_id"}=1;
							#$seen_comb{"$SEQ_IDs{$chr}[1][$i].$cluster_id"}=1;
						}
						$overl++;	
					}
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Gains: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return ($cluster_id);
}

sub find_overlaps_inversions_A {# (-)LINKER(+)
	my $cnt=0;
	my $overl=0;
	my $ident=0;
	my $cluster_id=0;
	my %seen_comb;
	#%HighStringency=();	
	print STDERR "Looking for overlaps (agreeing inversions)... \n";
	foreach my $chr (sort keys %CHR) {
		print STDERR "$chr ";
		next unless ($STARTs{$chr}[2][1]);
		for (my $i=0; $i<@{$STARTs{$chr}[2]}-1; $i++) {
		#print STDERR "$IDs{$chr}[2][$i]...\n";
			next unless ($STRAND_As{$chr}[2][$i] eq "-");
			next if ($number_of_calls{$IDs{$chr}[2][$i]}>1);
			for (my $y=$i+1; $y<@{$STARTs{$chr}[2]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[2][$y]}>1);
				next unless ($STRAND_As{$chr}[2][$y] eq "-");
				#1. assess whether distances between starts ends believable
				if ( (abs($STARTs{$chr}[2][$i]-$STARTs{$chr}[2][$y]) <=$max_discrepancy) and (abs($ENDs{$chr}[2][$i]-$ENDs{$chr}[2][$y]) <= $max_discrepancy)) {#XXX check, is discrepancy correct, as sizes get incremently larger?
			    	    #2. -- assess overlap (one must encompass other!) #XXX
				    if (($STARTs{$chr}[2][$i] <= $STARTs{$chr}[2][$y] && $STARTs{$chr}[2][$y] <= $ENDs{$chr}[2][$i]) or ($STARTs{$chr}[2][$i]>=$STARTs{$chr}[2][$y] && $STARTs{$chr}[2][$i] <= $ENDs{$chr}[2][$y])) {
					#.3. -- does inversion fit?
					my ($biggest_start_b) = sort {$b<=>$a} ($STARTbs{$chr}[2][$i],$STARTbs{$chr}[2][$y]);
					my ($smallest_end_b) = sort {$a<=>$b} ($ENDbs{$chr}[2][$i],$ENDbs{$chr}[2][$y]);
					if ($biggest_start_b<$smallest_end_b) {
						#print STDERR "biggest_start_b=$biggest_start_b; smallest_end_b=$smallest_end_b\n";
						if (($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y]) or ($STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y]) or ($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y]) or ($ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y])) {
							print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n"  if ($output_identical_and_near_identical_reads); 
							$ident++;
						} else {
							print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n" if ($no_linkage_clustering); 
							#-- sort into cluster
							my $match = 0;
							foreach my $cluster_id (@{$SLclust[2]}) {
								if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
									$match=1;
									#next;
								} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
									$match=1;
									#next;
								}
							}
							if (!$match) {
								$cluster_id++;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
								push (@{$SLclust[2]}, $cluster_id);
							}
							#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[2][$i] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[2][$i].$cluster_id"}));
							#$HighStringency{$cluster_id}++ if ($SEQ_IDs{$chr}[2][$y] >= $min_HS_id && !exists ($seen_comb{"$SEQ_IDs{$chr}[2][$y].$cluster_id"}));
							#$seen_comb{"$SEQ_IDs{$chr}[2][$i].$cluster_id"}=1;
							#$seen_comb{"$SEQ_IDs{$chr}[2][$y].$cluster_id"}=1;
															
						}
						$overl++;	
					}
				    }
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Inversions: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return ($cluster_id);
}
sub find_overlaps_inversions_B { # (+)LINKER(-)
	my $cnt=0;
	my $overl=0;
	my $ident=0;
	my $cluster_id=0;
	my %seen_comb;
	print STDERR "Looking for overlaps (agreeing inversions)... \n";
	foreach my $chr (sort keys %CHR) {
		print STDERR "$chr ";
		next unless ($STARTs{$chr}[2][1]);
		for (my $i=0; $i<@{$STARTs{$chr}[2]}-1; $i++) {
		#print STDERR "$IDs{$chr}[2][$i]...\n";
			next unless ($STRAND_As{$chr}[2][$i] eq "+");
			next if ($number_of_calls{$IDs{$chr}[2][$i]}>1);
			for (my $y=$i+1; $y<@{$STARTs{$chr}[2]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[2][$y]}>1);
				next unless ($STRAND_As{$chr}[2][$y] eq "+");
				#1. assess whether distances between starts ends believable
				if ( (abs($STARTs{$chr}[2][$i]-$STARTs{$chr}[2][$y]) <=$max_discrepancy) and (abs($ENDs{$chr}[2][$i]-$ENDs{$chr}[2][$y]) <= $max_discrepancy)) {#XXX check, is discrepancy correct, as sizes get incremently larger?
			    	    #2. -- assess overlap (one must encompass other!) #XXX
				    if (($STARTs{$chr}[2][$i] <= $STARTs{$chr}[2][$y] && $STARTs{$chr}[2][$y] <= $ENDs{$chr}[2][$i]) or ($STARTs{$chr}[2][$i]>=$STARTs{$chr}[2][$y] && $STARTs{$chr}[2][$i] <= $ENDs{$chr}[2][$y])) {
					#.3. -- does inversion fit?
					my ($biggest_start_b) = sort {$b<=>$a} ($STARTbs{$chr}[2][$i],$STARTbs{$chr}[2][$y]);
					my ($smallest_end_b) = sort {$a<=>$b} ($ENDbs{$chr}[2][$i],$ENDbs{$chr}[2][$y]);
					if ($biggest_start_b<$smallest_end_b) {
						if (($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y]) or ($STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y]) or ($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y]) or ($ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y])) {
							print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n"  if ($output_identical_and_near_identical_reads); 
							$ident++;
						} else {
							print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n" if ($no_linkage_clustering); 
							#-- sort into cluster
							my $match = 0;
							foreach my $cluster_id (@{$SLclust[2]}) {
								if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
									$match=1;
								} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]})) {
									$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
									$match=1;
								}
							}
							if (!$match) {
								$cluster_id++;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
								$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
								push (@{$SLclust[2]}, $cluster_id);
							}																					
						}
						$overl++;	
					}
				    }
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Inversions: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return ($cluster_id);
}
sub find_overlaps_complex { 
    my $cnt=0;
	my $overl=0;
	my $ident=0;
	my $cluster_id=0;
	my %seen_comb;
	print STDERR "Looking for overlaps (agreeing complex variants)... \n";
	foreach my $chr (sort keys %CHR) {
		print STDERR "$chr ";
		next unless ($STARTs{$chr}[3][1]);
		for (my $i=0; $i<@{$STARTs{$chr}[3]}-1; $i++) {
			next if ($number_of_calls{$IDs{$chr}[3][$i]}>1);
			for (my $y=$i+1; $y<@{$STARTs{$chr}[3]}; $y++) {
				next if ($number_of_calls{$IDs{$chr}[3][$y]}>1);
				#1. assess whether distances between starts ends believable
				if ( (abs($STARTs{$chr}[3][$i]-$STARTs{$chr}[3][$y]) <=$max_discrepancy) and (abs($ENDs{$chr}[3][$i]-$ENDs{$chr}[3][$y]) <= $max_discrepancy)) {#XXX check, is discrepancy correct, as sizes get incremently larger?
				#print STDERR "(abs($STARTs{$chr}[3][$i]-$STARTs{$chr}[3][$y]) <=$max_discrepancy and abs($ENDs{$chr}[3][$i]-$ENDs{$chr}[3][$y])) <= $max_discrepancy)\n";
				#2. -- assess overlap (one must encompass other!) #XXX
					if (($STARTs{$chr}[3][$i] <= $STARTs{$chr}[3][$y] && $STARTs{$chr}[3][$y] <= $ENDs{$chr}[3][$i]) or ($STARTs{$chr}[3][$i]>=$STARTs{$chr}[3][$y] && $STARTs{$chr}[3][$i] <= $ENDs{$chr}[3][$y])) {
					#.3. -- does inversion fit?
						my ($biggest_start_b) = sort {$b<=>$a} ($STARTbs{$chr}[3][$i],$STARTbs{$chr}[3][$y]);
						my ($smallest_end_b) = sort {$a<=>$b} ($ENDbs{$chr}[3][$i],$ENDbs{$chr}[3][$y]);
						if ($biggest_start_b<$smallest_end_b) {
							#print STDERR "biggest_start_b=$biggest_start_b; smallest_end_b=$smallest_end_b\n";
							#print STDERR "biggest_start_b=$biggest_start_b; smallest_end_b=$smallest_end_b  ($STARTs{$chr}[3][$i] $STARTbs{$chr}[3][$i] $ENDbs{$chr}[3][$i]  $ENDs{$chr}[3][$i] |  $STARTs{$chr}[3][$y] $STARTbs{$chr}[3][$y] $ENDbs{$chr}[3][$y]  $ENDs{$chr}[3][$y] )\n";
							
							if (($STARTs{$chr}[3][$i] == $STARTs{$chr}[3][$y] and $ENDs{$chr}[3][$i] == $ENDs{$chr}[3][$y]) or ($STARTbs{$chr}[3][$i] == $STARTbs{$chr}[3][$y] and $ENDbs{$chr}[3][$i] == $ENDbs{$chr}[3][$y]) or ($STARTs{$chr}[3][$i] == $STARTs{$chr}[3][$y] and $STARTbs{$chr}[3][$i] == $STARTbs{$chr}[3][$y]) or ($ENDs{$chr}[3][$i] == $ENDs{$chr}[3][$y] and $ENDbs{$chr}[3][$i] == $ENDbs{$chr}[3][$y])) {
								print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[3][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[3][$y]} ($i/$y $STARTs{$chr}[3][$i]/$STARTs{$chr}[3][$y] $ENDs{$chr}[3][$i]/$ENDs{$chr}[3][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[3][$i]/$PREDICTED_EVENT_SIZE{$chr}[3][$y])\n\n"  if ($output_identical_and_near_identical_reads);
							$ident++;
							} else {
								print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[3][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[3][$y]} ($i/$y $STARTs{$chr}[3][$i]/$STARTs{$chr}[3][$y] $ENDs{$chr}[3][$i]/$ENDs{$chr}[3][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[3][$i]/$PREDICTED_EVENT_SIZE{$chr}[3][$y])\n\n" if ($no_linkage_clustering);
								#-- sort into cluster
								my $match = 0;
								foreach my $cluster_id (@{$SLclust[3]}) {
									if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[3][$i]})) {
										$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]}=1;
										$match=1;
									} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]})) {
										$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$i]}=1;
										$match=1;
									}
								}
								if (!$match) {
									$cluster_id++;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$y]}=1;
									$SLclust_hash{$cluster_id}{$IDs{$chr}[3][$i]}=1;
									push (@{$SLclust[3]}, $cluster_id);
								}
								}
							$overl++;
						} else {
							#print STDERR "biggest_start_b=$biggest_start_b; smallest_end_b=$smallest_end_b  ($STARTs{$chr}[3][$i] $STARTbs{$chr}[3][$i] $ENDbs{$chr}[3][$i]  $ENDs{$chr}[3][$i] |  $STARTs{$chr}[3][$y] $STARTbs{$chr}[3][$y] $ENDbs{$chr}[3][$y]  $ENDs{$chr}[3][$y] )\n";

							#die;
							next; #goes over breakpoint
						}
					}
				}
				$cnt++;
			}
		}
	}
	print STDERR " ->Complex: $overl overlaps ($ident of which are due to identical matches) in $cnt comparisons...\n";
	return ($cluster_id);
}

sub find_overlaps_translocations { # (+)LINKER(-)
    my $cnt=0;
    my $overl=0;
	my $ident=0;
	my $cluster_id=0;
	print STDERR "Looking for overlaps (agreeing translocations)... \n";
	foreach my $key (sort keys %TRANSLOC_SEEN) {
		my ($CHR_A, $CHR_B) = split (/\./, $key);
		my $chr;
		next unless ($T_ASSIGN{$key}[1]);
		print STDERR "$key: @{$T_ASSIGN{$key}}\n";
		for (my $i=0; $i<@{$T_ASSIGN{$key}}-1; $i++) {
			next if ($number_of_calls{$T_IDs{$key}[$i]}>1);
			for (my $y=$i+1; $y<@{$T_ASSIGN{$key}}; $y++) {
				next if ($number_of_calls{$T_IDs{$key}[$y]}>1);
				#1. assess whether distances between starts ends believable
				my ($start_interv_1, $end_interv_1, $start_interv_2, $end_interv_2);
				if ($T_ASSIGN{$key}[$y] == $T_ASSIGN{$key}[$i]) {
					($start_interv_1, undef, undef, $end_interv_1) = sort {$a<=>$b} ($T_STARTs_A{$key}[$i], $T_STARTs_A{$key}[$y], $T_ENDs_A{$key}[$i], $T_ENDs_A{$key}[$y]);
					($start_interv_2, undef, undef, $end_interv_2) = sort {$a<=>$b} ($T_STARTs_B{$key}[$i], $T_STARTs_B{$key}[$y], $T_ENDs_B{$key}[$i], $T_ENDs_B{$key}[$y]);
				} else {
					($start_interv_1, undef, undef, $end_interv_1) = sort {$a<=>$b} ($T_STARTs_A{$key}[$i], $T_STARTs_B{$key}[$y], $T_ENDs_A{$key}[$i], $T_ENDs_B{$key}[$y]);
                    ($start_interv_2, undef, undef, $end_interv_2) = sort {$a<=>$b} ($T_STARTs_B{$key}[$i], $T_STARTs_A{$key}[$y], $T_ENDs_B{$key}[$i], $T_ENDs_A{$key}[$y]);
				}
				next if ($end_interv_1-$start_interv_1+1 > $max_discrepancy or $end_interv_2-$start_interv_2+1 > $max_discrepancy);
				#if ($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y]) or ($STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y]) or ($STARTs{$chr}[2][$i] == $STARTs{$chr}[2][$y] and $STARTbs{$chr}[2][$i] == $STARTbs{$chr}[2][$y]) or ($ENDs{$chr}[2][$i] == $ENDs{$chr}[2][$y] and $ENDbs{$chr}[2][$i] == $ENDbs{$chr}[2][$y])) { #XXX-1
				#				print "warning-identity:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n"  if ($output_identical_and_near_identical_reads);
				#				$ident++;
				#			} else {
				#print "overlap:\n---------\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$i]}\n$NON_TRANSL_CALLs{$IDs{$chr}[2][$y]} ($i/$y $STARTs{$chr}[2][$i]/$STARTs{$chr}[2][$y] $ENDs{$chr}[2][$i]/$ENDs{$chr}[2][$y] predicted_sizes:$PREDICTED_EVENT_SIZE{$chr}[2][$i]/$PREDICTED_EVENT_SIZE{$chr}[2][$y])\n\n" if ($no_linkage_clustering);
				#-- sort into cluster
				#my $match = 0;
				#foreach my $cluster_id (@{$SLclust[2]}) {
				#	if (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]})) 
				#		$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
				#		$match=1;	
				#	} elsif (exists($SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]})) {
				#		$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
				#		$match=1;
				#	}
				#}
				#if (!$match) {
				#	$cluster_id++;
				#	$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$y]}=1;
				#	$SLclust_hash{$cluster_id}{$IDs{$chr}[2][$i]}=1;
				#	push (@{$SLclust[2]}, $cluster_id);
				#}			
				$overl++;
				#		
				#		
				#$cnt++;
				print STDERR "$CHR_A $CHR_B $start_interv_1, $end_interv_1, $start_interv_2, $end_interv_2 [need exclude redundnacies! re-active XXX-1]\n"; 
			}
		}
	}
	print STDERR " ->Translocations: $overl overlaps ...\n";
	return ($cluster_id);
}

sub parse_blat_lookup_table {
	if ($blat_organism eq "mouse") {
		print STDERR "lookup table not done yet for mouse! skipped...\n";
		return;
	}
	my $file = "~/Sauron/Projects/PairedEnd/Data/blat_454_PEread_lookup_table_ncbi36.txt.gz";
	open FH, "gunzip -c $file |";
	print STDERR "Parsing $file ... ";
	my $cnt=0;
	while (<FH>) {
		chomp;
		next unless ($_);
		my ($id, $hq_score) = split;
		if (!defined ($hq_score)) {
			print STDERR "wrong line in $file!\n";
			die;
		}
		$PREVIOUSLY_SUCCESSFULLY_EVALUATED{$id} = $hq_score if ($hq_score);
		$PREVIOUSLY_EVALUATED{$id}=2;#pretend A and B have been seen
		$cnt++;
	}
	close FH;
	print STDERR "Blat results for $cnt reads parsed...\n";
	die unless ($cnt);
}
																																																																																																																																																																																																																																																																																																																																																																																																																																																														


