#!/usr/bin/perl

use warnings;
use strict;

my $n_arg = @ARGV;
if ($n_arg == 0) {
    print STDERR "Please provide input file name.\n";
    exit;
}
my $infile = $ARGV[0];

if ($n_arg == 1) {
    print STDERR "Please provide output file name.\n";
    exit;
}
my $outfile = $ARGV[1];

if ($n_arg == 2) {
    print STDERR "Please provide mapping file name.\n";
    exit;
}
my $mapfile = $ARGV[2];

my $command = "blat_executable -minMatch=1 value -f value_of_megablastDatabase ".$infile." ".$outfile;

system($command);

if (!open(FILE,$outfile)) {
    print STDERR "Can't read file '",$outfile,"'.\n";
    exit;
}

if (!open(MAPFILE,">$mapfile")) {
    print STDERR "Can't write to file '",$mapfile,"'.\n";
    exit;
}

my $line = "";
while ($line = <FILE>) {
    my @words = split(/\s/,$line);
    my $n_words = @words; if ($n_words < 17) { next; }
    if (!($words[0] =~ /\d+/)) { next; } # first word is number
    my $match = $words[0] + $words[2];
    my $score = $words[1] + $words[5] + $words[7]; # mismatch + q_gap + t_gap
    if ($score > 0.2*$match) { next; } # 80% sequence identity
    print MAPFILE "'",$words[13],"'=='",$words[8],$words[9],"' ";
    if ($words[8] eq "+") {
	print MAPFILE "(",$words[15]," ",$words[11]," ",$words[16]," ",$words[12],") ";
    } else {
	print MAPFILE "(",$words[15]," ",$words[12]," ",$words[16]," ",$words[11],") ";
    }
    print MAPFILE $score,"\n";
}

close(FILE);
close(MAPFILE);

exit;

# calc psl aln score, http://genome.ucsc.edu/FAQ/FAQblat.html#blat6
sub pslScore # Return score for psl.
{
    my $sizeMul = 3; #pslIsProtein(psl) ? 3 : 1;

    my @fs = @_;
    my ($matches, $misMatches, $repMatches, $qNumInsert, $tNumInsert) =
	($fs[0],
	 $fs[1], $fs[2], $fs[4], $fs[6]);
    my $score = $sizeMul * ($matches + ( $repMatches>>1 )) - $sizeMul *
	$misMatches - $qNumInsert - $tNumInsert;
    
    return $score;
}
