#!/usr/bin/env python

import gzip, imp, optparse, os, sys

stdE, stdO = sys.stderr, sys.stdout

#-- sub						
def parse_infile(infile):
    if infile.endswith('.gz'):
        f = gzip.GzipFile(infile)
    else:
        f = open(infile)

    print >>stdE, 'Parsing %s ... '%infile

    col_lab = f.next()[:-1].split('\t')
    col_lab = dict([(col_lab[x], x) for x in xrange(len(col_lab))])

    All_hits, cnt, seen = {}, 0, {}
    vars = ['CHR', 'CIRCLE_ID', 'END', 'LENGTH', 'PERC_SEQID_UNWEIGHED', 'READ_ID', 'START', 'STRAND']
    class peData: pass # python idiom to create a structure.

    for l in f:
        l = l.strip()
        if not l or l in seen: continue
        seen[l] = 1
        cols, ped = l.split(), peData()
        for v in vars: exec('ped.%s = cols[col_lab["%s"]]'%(v, v))
        All_hits.setdefault(ped.CIRCLE_ID, {}).setdefault(ped.READ_ID, []).append(ped)
        cnt += 1

    print >>stdE, ' %d entries parsed (%d circle-IDs).'%(cnt, len(All_hits))
    f.close()

    return All_hits


def outLabelPed(label, ped, outfile):
    print >>outfile, '\t'.join([ped.CIRCLE_ID, ped.READ_ID,
                                ped.CHR, ped.START, ped.END, ped.STRAND, ped.PERC_SEQID_UNWIEGHED, ped.LENGTH,
                                '['+label+']'])


def discard_entire_circle(label, CIRCLE_ID, outfile):
    print >>outfile, CIRCLE_ID+'\t'+label

def prepare_and_run_placement_algorithm(All_hits, discards):
    print >>stdE, 'Preparing generation of output...'
    cnt = 0
    cids = All_hits.keys()
    cids.sort()
    for CIRCLE_ID in cids:
        ahd = All_hits[CIRCLE_ID]
        cnt += 1
        reads = ahd.keys()
        reads.sort()
        #-- 1. test number of reads per circle, should be 2!
        if len(reads) > 2:
            print >>stdE, 'error #3: more than 2 reads per circle!'
            sys.exit(1)
        elif len(reads) < 2:
            discard_entire_circle ('only one read matches the genome', CIRCLE_ID, discards)
            continue
		
        #-- 2. find longest and best perc_id for both reads
        best_length, best_seqid = [0,0], [0,0] #initialize
        for i in range(2):
            for ped in ahd[reads[i]]:
                l, psu = float(ped.LENGTH), float(ped.PERC_SEQID_UNWEIGHED)
                if l == best_length[i]:
                    best_length[i] += 0.001 #if two are equal, no best LENGTH should be rewarded any more!!
                elif l > best_length[i]:
                    best_length[i] = l

                if psu == best_seqid[i]:
                    best_seqid[i] += 0.001 #if two are equal, no best SEQID should be rewarded any more!!
                elif psu > best_seqid[i]:
                    best_seqid[i] = psu
		
        #-- 3. compare all against all within a circle and calculate placement score
        PLACEMENT_SCORE = {}
        for ped0 in ahd[reads[0]]:
            for ped1 in ahd[reads[1]]:
                PLACEMENT_SCORE.setdefault(placement(ped0, ped1, best_seqid, best_length), []).append((ped0, ped1))

        #-- 4. output pair(s) with highest placement score(s) [routine is stopped after first iteration]
        pScores = PLACEMENT_SCORE.keys()
        pScores.sort(reverse=True)
        for pScore in pScores:
            for ped0, ped1 in PLACEMENT_SCORE[pScore]:
                print '\t'.join([CIRCLE_ID, pScore,
                                 ped0.CHR, ped0.START, ped0.END, ped0.STRAND, ped0.PERC_SEQID_UNWEIGHED, ped0.LENGTH,
                                 ped1.CHR, ped1.START, ped1.END, ped1.STRAND, ped1.PERC_SEQID_UNWEIGHED, ped1.LENGTH])
            break

    print >>stdE, '%d circle_ids processed and output written...'%cnt


def placement(ped0, ped1, best_seqid, best_length):
    if float(ped0.LENGTH) > best_length[0]: #unnecessary consistency check
        print >>stdE,'Failed consistency check.'
        print >>stdE, ped0.LENGTH, ped1.LENGTH, best_seqid, best_length
        sys.exit(1)
        
    score = 0.0

    #-- score (+1) for best seq_ids per end
    score += float(ped0.PERC_SEQID_UNWEIGHED) == best_seqid[0]
    score += float(ped1.PERC_SEQID_UNWEIGHED) == best_seqid[1]
	
    #-- score (+1) for best lengths per end
    score += float(ped0.LENGTH) == best_length[0]
    score += float(ped1.LENGTH) == best_length[1]

    #-- score (+Delta) for seq_ids >= Threshold (high quality) per end
    score += config.highQualityScoreDelta*(float(ped0.PERC_SEQID_UNWEIGHED) >= config.highQualityThreshold)
    score += config.highQualityScoreDelta*(float(ped1.PERC_SEQID_UNWEIGHED) >= config.highQualityThreshold)
	
    #-- score (+2) for concordant insert size (per pair)
    #-- size estimation!
    pos = [int(ped0.START), int(ped0.END), int(ped1.START), int(ped1.END)]
    pos.sort()
    prange = pos[3] - pos[0]
    score += 2*(ped0.CHR == ped1.CHR and prange >= config.min_concordant and prange <= config.max_concordant)

    #-- score (+1) for concordant orientation (per pair)
    score += ped0.STRAND == ped1.STRAND and ped0.CHR == ped1.CHR

    return '%4.2f'%(score/(7.+2*config.highQualityScoreDelta)) #maximum = 1

#-- main

ops = optparse.OptionParser()
ops.description = 'Determine placement scores for the mapped paired ends in infile.'
ops.add_option('-d', '--discards',
               help='write discarded PairedEnds (e.g. \'2 identical placement scores\') to FILE [%default]',
               default='discarded-placements-ends-%d.txt'%os.getpid(), metavar='FILE')
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.set_usage('%prog [options] infile')

oArgs, pArgs = ops.parse_args()

if len(pArgs) != 1:
    ops.print_help()
    sys.exit(1)
infile = pArgs[0]

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.

#-- open outfile_discarded
discards = open(oArgs.discards, 'w')

print 'CIRCLE_ID\tPLACEMENT_SCORE\tCHR_A\tSTART_A\tEND_A\tSTRAND_A\tPERC_SEQID_UNWEIGHED_A\tLENGTH_A\tCHR_B\tSTART_B\tEND_B\tSTRAND_B\tPERC_SEQID_UNWEIGHED_B\tLENGTH_B'
All_hits = parse_infile(infile)
prepare_and_run_placement_algorithm(All_hits, discards)

discards.close()

