#!/usr/bin/env python

# Alternate builds are now handled via separate runs with their own directory structure (not interleaved as before).

import imp, optparse, os, re, resource, sys, time

stdE = sys.stderr

def timeStamp(txt): print >>stdE, txt, time.strftime('%a %b %d %H:%M:%S %Y'), time.time()

ops = optparse.OptionParser()
ops.description = 'Run paired end pipeline on a batch of reads'
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.add_option('-s', '--startStep', help='start processing with this step', type=int)
ops.set_usage('%prog [options] infile batchStart batchEnd')

oArgs, pArgs = ops.parse_args()

if len(pArgs) != 3:
    ops.print_help()
    sys.exit(1)
infile, start, end = pArgs

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
userConfigOpt = ''
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.
    userConfigOpt = '-c '+os.path.abspath(oArgs.config)      # for relay to sub commands

#-- main
timeStamp('At Start')

label = start+'-'+end
outfile = re.sub('\.[0-9]+\.fa(\.gz)?\Z', '', os.path.basename(infile))

#-- 1. map linkers and extract paired reads
timeStamp('At 1')
disfile1 = config.tmp_dir+'/discard/'+outfile+'-linkermapped-'+label+'.fa_discard'

if config.linkermappedDir:
    # we already have output in a linkermapped directory (with '/fa' if needed ---
    # no extra '/fa/' add here!), following, we assume, the same
    # naming convention as below.
    # THIS LOGIC BRANCH NOT TESTED!
    outfile1 = config.linkermappedDir+'/'+outfile+'-linkermapped-'+label+'.fa'
else:
    outfile1 = config.tmp_dir+'/fa/'+outfile+'-linkermapped-'+label+'.fa'
    run = ' '.join([config.scriptPath+'/map_linkers_and_split_raw_reads.py', userConfigOpt, infile, disfile1, '>', outfile1])
    if oArgs.startStep <= 1:
        print >>stdE, run+' ...\n'
        os.system(run)

#--  2. run megablast (initial recruitment)
timeStamp('At 2')
outfile2 = config.tmp_dir+'/megablast/'+outfile+'-'+label+'.megablast'
run = ' '.join([config.scriptPath+'/runMegablast.py', userConfigOpt, outfile1, '>', outfile2])
if oArgs.startStep <= 2:
    print >>stdE, run+' ...'
    os.system(run)

#-- 3. run needleman-wunsch (re-align)
timeStamp('At 3')
outfile3 = config.tmp_dir+'/needle/'+outfile+'-'+label+'.needle'
run = ' '.join([config.scriptPath+'/megablastOut2Needle.py', userConfigOpt, outfile2, outfile1, '>', outfile3])
if oArgs.startStep <= 3:
    print >>stdE, run+' ...'
    os.system(run)

#-- 4. placement
timeStamp('At 4')
outfile4 = config.tmp_dir+'/placement/'+outfile+'-'+label+'.placemnt'
discarded4 = config.tmp_dir+'/discard/'+outfile+'-'+label+'.placemnt_discard'
run  = ' '.join([config.scriptPath+'/Hits2PlacementScore.py', userConfigOpt, '-d', discarded4, outfile3, '>', outfile4])
if oArgs.startStep <= 4:
    print >>stdE, run+' ...'
    os.system(run)

#-- 5. call of structural variants
timeStamp('At 5')
outfile5 = config.tmp_dir+'/call/'+outfile+'-'+label+'.call'
discarded5 = config.tmp_dir+'/discard/'+outfile+'-'+label+'.call_discard'
run = ' '.join([config.scriptPath+'/RetrieveStrucVariantsFromEndPairsWithPlacementScore.py',
                userConfigOpt, '-d', discarded5, '-l', outfile1, outfile4,'>', outfile5])
if oArgs.startStep <= 5:
    print >>stdE, run+' ...'
    os.system(run)

print >>stdE, 'DONE computing on NODE (%s-%s)!'%(start, end)
timeStamp('At Finish')
print >>stdE, 'Resource usage self:', resource.getrusage(resource.RUSAGE_SELF)
print >>stdE, 'Resource usage children:', resource.getrusage(resource.RUSAGE_CHILDREN)
print >>stdE, 'getpagesize:', resource.getpagesize()
