#!/usr/bin/env python

import gzip, imp, optparse, os, sys

stdE, stdO = sys.stderr, sys.stdout

def parse_read_parameters(linkers):
    print >>stdE, 'Excluding ... well actually not. parse_read_parameters called but function not defined.'


def parse_infile(infile, discards):
    if infile.endswith('.gz'):
        f = gzip.GzipFile(infile)
    else:
        f = open(infile)

    print >>stdE, 'Parsing %s ... '%infile

    col_lab = f.next()[:-1].split('\t')
    col_lab = dict([(col_lab[x], x) for x in xrange(len(col_lab))])

    cnt = 0
    for l in f:
        l = l.strip()
        if not l: continue
        if l.startswith('CIRCL') or l.startswith('Pars') or l.startswith('Prepar'): continue
        if 'processed' in l: continue
        MapPairedEnd2potentialVariant(col_lab, l.split(), discards)
        cnt += 1

    print >>stdE, ' %d entries parsed.'%cnt
    f.close()


def MapPairedEnd2potentialVariant(col_lab, cols, discards):
    class peData: pass # python idiom to create a structure.
    ped = peData()
    vars = ['CHR_A', 'CHR_B', 'CIRCLE_ID', 'END_A', 'END_B', 'LENGTH_A', 'LENGTH_B',
            'PERC_SEQID_UNWEIGHED_A', 'PERC_SEQID_UNWEIGHED_B', 'PLACEMENT_SCORE', 'START_A', 'START_B', 'STRAND_A', 'STRAND_B']
    for v in vars: exec('ped.%s = cols[col_lab["%s"]]'%(v, v))
    
    Exclude = {} # if parse_read_parameters were not a dummy function, then
                 # this might be (globally) set to something other than empty.
    if ped.CIRCLE_ID in Exclude:
        outLabelPed('low quality linker match', ped, discards)
    elif float(ped.PERC_SEQID_UNWEIGHED_A) < config.min_sequence_id_on_read \
             or float(ped.PERC_SEQID_UNWEIGHED_B) < config.min_sequence_id_on_read \
             or float(ped.PLACEMENT_SCORE) < config.min_placement_score:
        outLabelPed('low quality alignment or score', ped, discards)
    elif ped.CHR_B != ped.CHR_A: #-- filtering translocations
        if config.trans:
            outLabelPed('predicted translocation', ped, stdO)
        else:
            outLabelPed('filtered potentially spurious translocation', ped, discards)
    else: #-- inversions, deletions, insertions
        coord = [int(ped.START_A), int(ped.END_A), int(ped.START_B), int(ped.END_B)]
        coord.sort()

        #njc this could use data from just A or B --- is that intended?
        mapped_range = coord[-1] - coord[0]; #-- determine range of inversion (roughly equivalent mapping resolution)

        if mapped_range > (config.max_event_size+config.max_concordant):
            outLabelPed('>max_range', ped, discards)
        elif ped.STRAND_A != ped.STRAND_B:   #-- filtering inversions	
            outLabelPed('predicted inversion', ped, stdO)
        elif mapped_range > config.max_concordant:  #-- $CHR_A eq $CHR_B, concordant orientation, probable deletion in GM15110
            outLabelPed('predicted deletion in sample', ped, stdO)
        elif mapped_range < config.min_concordant:  #-- $CHR_A eq $CHR_B, concordant orientation, probable insertion in GM15110		
            outLabelPed('predicted insertion in sample', ped, stdO)
        else:
            outLabelPed('concordant', ped, discards)

def outLabelPed(label, ped, outfile):
    print >>outfile, '\t'.join([ped.CIRCLE_ID, ped.PLACEMENT_SCORE,
                                 ped.CHR_A, ped.START_A, ped.END_A, ped.STRAND_A, ped.PERC_SEQID_UNWEIGHED_A, ped.LENGTH_A,
                                 ped.CHR_B, ped.START_B, ped.END_B, ped.STRAND_B, ped.PERC_SEQID_UNWEIGHED_B, ped.LENGTH_B,
                                 '['+label+']'])


ops = optparse.OptionParser()
ops.description = 'Extract structure variants from the mapped paired ends in infile.'
ops.add_option('-l', '--linkers', help='linker mapped fasta data in FILE', metavar='FILE')
ops.add_option('-d', '--discards',
               help='write discarded PairedEnds (e.g. \'concordant\' paired-ends, unallowed translocations, etc.) to FILE [%default]',
               default='discarded-paired-ends-%d.txt'%os.getpid(), metavar='FILE')
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.set_usage('%prog [options] infile')

oArgs, pArgs = ops.parse_args()

if len(pArgs) != 1:
    ops.print_help()
    sys.exit(1)
infile = pArgs[0]

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.

#-- open outfile_discarded
discards = open(oArgs.discards, 'w')

#-- main
print 'CIRCLE_ID\tPLACEMENT_SCORE\tCHR_A\tSTART_A\tEND_A\tSTRAND_A\tPERC_SEQID_UNWEIGHED_A\tLENGTH_A\tCHR_B\tSTART_B\tEND_B\tSTRAND_B\tPERC_SEQID_UNWEIGHED_B\tLENGTH_B\tCALL'
#njc discards file doesn't seem to get a header line.

if oArgs.linkers: parse_read_parameters(oArgs.linkers)
parse_infile(infile, discards)

discards.close()
