#!/usr/bin/perl

use warnings;
use strict;

my @clusters = ();
unless (@ARGV) {
	
print STDERR "give files to combine as arguments; merging will be based on identical reads...\n";
	print STDERR "if last argument = \'blat\', then only clusters with blat score of 2 will be included...\n";
	print STDERR "-> ignores T1/T2 clusters...\n";
	exit
}

my $blat_option = 0;
if ($ARGV[-1] eq "blat") {
	pop @ARGV;
	$blat_option = 1;
}

my @infiles = (@ARGV);

foreach my $file (@infiles) {
    if (!open(FILE,$file)) {
	print STDERR "Can't open file '",$file,"'\n";
	next;
    }
    while (my $line = <FILE>) {
	
	if (substr($line,0,2) eq "#>") {
	    if ($line =~ />.*\//) {#excludes T1/T2 clusters
	    	print STDERR "ignoring $line ...\n";
		next;
	    }
	    my @new_cluster = ($line);
	    my $blat_score = 0;
	    while ($line = <FILE>) {
		if (length($line) <= 1)       { last; }
		if (substr($line,0,1) eq "#") { last; }
		push(@new_cluster,$line);
	        if ($line =~ /\[Distance_score\([0-9]+bp\)\=2\]/) {
			#print STDERR "good match: $line\n";
			$blat_score += 2;
		}
	    }
	    my $old_cluster = findCluster(\@clusters,\@new_cluster);
	    if (!$old_cluster) {
		if ($blat_option) {
			push(@clusters,\@new_cluster) if ($blat_score >= 2);
		} else {
			push(@clusters,\@new_cluster);
		}
	    } else {
#		print STDERR "Redundant\n"
		joinClusters($old_cluster,\@new_cluster);
		my $reported_cl_1 = $new_cluster[0];
		my $reported_cl_2 = $$old_cluster[0];
		chomp ($reported_cl_1); chomp ($reported_cl_2);
		print STDERR "joined $reported_cl_1 with $reported_cl_2 in \"$file\"...\n";
	    }
	}
    }
    close(FILE);

}

my $n = @clusters;
print STDERR "Number of clusters (after merging) is ",$n,"\n";
my $index = 1;
foreach my $clus (@clusters) {
    my $tmp = "Cluster ".$index;
    foreach my $l (@$clus) {
	if (substr($l,0,1) eq "#") { $l =~ s/Cluster\s\d+/$tmp/; }
	print $l;
    }
    print "\n\n";
    $index++;
}
exit;


sub findCluster
{
    my ($old_clusters,$new_cluster) = @_;
    my $ret;
    foreach my $old_c (@$old_clusters) {
	foreach my $new_l (@$new_cluster) {
	    if (substr($new_l,0,1) eq "#") { next; }
	    foreach my $old_l (@$old_c) {
		if (substr($old_l,0,1) eq "#") { next; }
		if ($old_l eq $new_l) {
		    if ($ret && $ret != $old_c) {
			print STDERR "Ambiguous match (clusters $ret $old_c)\n";
			return $ret;
		    }
		    $ret = $old_c;
		    last;
		}
	    }
	}
    }
    return $ret;
}

sub joinClusters
{
    my ($old_cluster,$new_cluster) = @_;
    foreach my $new_l (@$new_cluster) {
	if (substr($new_l,0,1) eq "#") { next; }
	my $same_line = "false";
	foreach my $old_l (@$old_cluster) {
	    if (substr($old_l,0,1) eq "#") { next; }
	    if ($old_l eq $new_l) {
		$same_line = "true";
		last;
	    }
	}
	if ($same_line eq "false") {
	    push(@$old_cluster,$new_l);
	}
    }
}
