# Set the default script path assuming this file lives in that directory.
import os.path
scriptPath = os.path.abspath(os.path.dirname(__file__))

# Directory containing reads after linker mapping. If linker mapping
# hasn't yet been done, leave this empty and the processing will
# include a step to do the linker mapping.
linkermappedDir = ''

# File containing the linker sequence.
linker_file = scriptPath+'/Data/linker_sequence.fa'
# An end must be at least this long to be kept after the linker is removed.
min_length_of_mapped_reads = 15
# Minimal acceptable % seq identity for a linker match.
min_linker_mapped_seq_id = 90
# Minimal acceptable len for a linker match.
min_linker_mapped_span = 40

# Parameters for pruning megablast output.
max_duplicates_per_score = 30
max_scores_retained_per_seq = 20

# Factor for extending megablast hits prior to running water.
expectedGapSpanMultiplier = 1.2

# Acceptable range for "normal" paired end separation.
min_concordant = 1174
max_concordant = 7694

# Cutoff for acceptable structure variant range.
max_event_size = 10000000
# Minimal acceptable placement score for an sv.
min_placement_score = 0.5
# Minimum sequence identity to qualify ends for an sv.
min_sequence_id_on_read = 90
# Placement score parameters.
highQualityThreshold = 97
highQualityScoreDelta = 1
# Predict translocations?
trans = True

# megablast program.
megablastExecutable = '/home2/jk377/bin/megablast'
# Input database for megablast.
megablastDatabase = '/home2/jk377/Data/Genomes/UCSC/Human_Build36_hg18_Mar2006/Human_Build36_hg18_Mar2006.fa'
#megablastDatabase = '/home2/jk377/Data/Genomes/Celera_Human/celera_human_all_sequences_relabeled.fa'
#megablastDatabase = '/home2/jk377/Data/Genomes/Prokaryotes/E.coliK12_genome_NC_000913.2/E.coliK12_genome_NC_000913.2.fa'
# Options for running megablast.
megablastOptions = '-D 0 -p 80 -s 11 -W 11'
#e coli: megablastOptions = '-p 80 -s 9 -v 7 -b 7 -W 12 -t 16' #(4^12)/2=8388608

# A file that maps from a chromosome id used in the megablast input
# database to the full path of the fasta format file that contains the
# sequence for the chromosome.  Each line of the map file has the
# format:
#
# MegablastDatabaseChromosomeId<<>>/full/path/to/chromosome/file
#
# The following is an example of how to generate such a map file. This
# example assumes that all of the chromosome file names begin with
# 'chr' and end with '.fa' and are located in the same directory, more
# generally "/foo/bar/chr*.fa" could be replaced with an explicit
# listing of files:
#
# for f in /foo/bar/chr*.fa; do echo "$(head -1 $f | gawk '{print substr($1, 2)}')<<>>$f"; done > MapFile
#
chrIdToFileMap = '/You/Need/To/Specify/The/Mapping/File'

# water program.
waterExecutable = '/home2/jk377/bin/water'
# water is run in two contexts, one for linker mapping ... 
waterLinkerMapDatafile = scriptPath+'/Data/DNA_PAIRED_END'
waterLinkerMapOptions = '-gapopen 20 -gapextend 1'
# ... and one for megablast hit refinement.
waterMBRefineDatafile = waterLinkerMapDatafile
waterMBRefineOptions = waterLinkerMapOptions

# Specify the directory or directories where Python utilities (such as
# getSeq.py) can be found.
PythonUtilitiesPath = [scriptPath+'/PythonUtilities/']

# Size of a bundle of reads.
package_size = 100
# Prepare for batch execution?
runBatch = True

# output files are written to subdirectories of this direcrtory.
tmp_dir = './tmp'
