#!/usr/bin/env python

import os, imp, optparse, re, subprocess, sys

stdE = sys.stderr

#-- defaults;
needle_tmp_file = '/tmp/map_needle_tmp_file.%d'%os.getpid()

#-- subs
def parse_infile(infile, discards):
    if infile.endswith('.gz'):
        f = gzip.GzipFile(infile)
    else:
        f = open(infile)

    print >>stdE, 'Parsing %s ....'%infile

    # def lines vary so need a better regular expression and better error handling. (njc)
    # ordering important!
    # perhaps make make def line re a config parameter?
    defLineRes = [(re.compile(r'^>[0-9a-zA-Z\-\+\_\:]+.*uaccno\=(.*)'), 1),
                  (re.compile(r'^>([0-9a-zA-Z\-\+\_\:]+)'), 1)]

    cnt, id, seq = 0, '', ''
    for l in f:
        l = l.strip()
        if not l: continue

        if l[0] == '>':
            for cre, gx in defLineRes:
                mo = cre.search(l)
                if mo == None: continue
                if seq: map_and_needle(id, seq, discards)
                id, seq = mo.group(gx), ''
                cnt += 1
                break
            else:
                print >>stdE, 'Cannot parse def line: '+l
        else:
            seq += l

    if seq: map_and_needle(id, seq, discards)
    print >>stdE, 'Mapped %d sequences'%cnt
    f.close()

# regular expression to find homopolymers.
hpre = re.compile('(.*)GT{1,4}G{1,4}A{1,4}C{1,4}G{0,3}A{2,5}G{2,5}T{2,5}G{0,3}A{1,4}T{1,4}C{0,3}A{2,5}C{2,5}T{2,5}C{0,3}G{1,4}T{1,4}C{1,4}A{1,4}C(.*)')

def map_and_needle(id, seq, discards):
    #-- prepare needle
    open(needle_tmp_file, 'w').write('>chro_needle_tmp\n%s\n'%seq) #frag from human chromosome

    start_match, end_match, match_length, sequence_identity = run_water('+')
    use = False

    # None or INTEGER for start/end ##########################3
    if start_match and end_match:
        start_match, end_match = int(start_match), int(end_match)
        readA, readB = seq[:start_match-1], seq[end_match:]
        if len(readA) >= config.min_length_of_mapped_reads and len(readB) >= config.min_length_of_mapped_reads:
            linkerMatchLen = end_match - start_match + 1
            if float(sequence_identity) >= config.min_linker_mapped_seq_id and linkerMatchLen >= config.min_linker_mapped_span:
                temp = '>%s.%s_%%s (linker=%d-%d[%dbp;%s%%%%)\n%%s'%(id, id, start_match, end_match, linkerMatchLen, sequence_identity)
                print temp%('A', readA)
                print temp%('B', readB)
                use = True

    if not use:
        #attempt mapping by regular expression (to find homopolymers)
        mo = hpre.search(seq)
        if mo:
            seqA, seqB = mo.groups()
            if len(seqA) >= config.min_length_of_mapped_reads and len(seqB) >= config.min_length_of_mapped_reads:
                print '>%s.%s_A (linker=matched_reg_expr)\n'%(id, id)+seqA
                print '>%s.%s_B (linker=matched_reg_expr)\n'%(id, id)+seqB
                use = True

    if not use:
        print >>discards, '%s discarded: mapped read<%d (e.g. too short, or linker not found)'%(id, config.min_length_of_mapped_reads)


def run_water(strand):
    cmd = ' '.join([config.waterExecutable,
                    needle_tmp_file,
                    config.linker_file,
                    config.waterLinkerMapOptions,
                    '-datafile '+config.waterLinkerMapDatafile,
                    'stdout'])
    if strand == '-': cmd += ' -sreverse2 TRUE'
    print >>stdE, cmd, '... '	
    p = subprocess.Popen(cmd, shell = True, stdout=subprocess.PIPE, stderr=open('/dev/null', 'w'))

    idre = re.compile(r'^# Identity.* ([0-9]+)/.*\(([0-9\.]+)%\)')
    start_match, end_match, match_length, sequence_identity = None, None, None, None
    for l in p.stdout:
        l = l.strip()
        mo = idre.search(l)
        if mo:
            match_length, sequence_identity = int(mo.group(1)), float(mo.group(2))
        elif l.startswith('chro_'):
            col = l.split()
            end_match = col[3]
            if start_match == None: start_match = col[1]

    p.wait()
    return start_match, end_match, match_length, sequence_identity

#-- main

# infile and only one (optional) argument: path to config data file. everything else is in there.
ops = optparse.OptionParser()
ops.description = 'Map linkers in reads.'
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.set_usage('%prog [options] file_with_raw_reads discarde_read_file')

oArgs, pArgs = ops.parse_args()

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.

# ordering a little strange here to try to accommodate the way jk generated a usage message in the original.
if len(pArgs) != 2:
    ops.print_help()
    print >>stdE, '\t\t-->uses the following linker sequence:',
    print >>stdE, 'currently assumes Ecoli-palindrome-linker: should compare both forward and reverse needle if not palindrome (probably best way: use BLAT!)!!'
    os.system('cat %s | grep -v "\>"'%config.linker_file)
    sys.exit(1)

infile, discards = pArgs[0], open(pArgs[1], 'w')

parse_infile(infile, discards)
discards.close()
try:    os.unlink(needle_tmp_file)
except: pass
