#!/usr/bin/env python

import gc, imp, optparse, os, re, subprocess, sys

gc.disable()

stdE = sys.stderr

##########
#-- subs #
##########
def parseMegablastOutput(fn):
    if fn.endswith('.gz'):
        import gzip
        f = gzip.open(fn)
    else:
        f = open(fn)
        
    sys.stderr.write('Parsing %s ... '%fn)
    cnt, mbByRead = 0, {}
    # jk's script as of 070324 seems to indicate we may need to trim trailing .fa's from some chromosome names...
    # we'll handle this downstream
    mbRe = re.compile(r'\'([^\']+)\'==\'([-+])([^\.]+)\.([^\']+)\' \((\d+) (\d+) (\d+) (\d+)\) (\d+)\n')
    for l in f:
        m = mbRe.match(l)
        if not m:
            sys.stderr.write('Warning: Don\'t understand: '+l)
            continue
        mbByRead.setdefault(m.group(4), []).append(m.groups())
        cnt += 1
    f.close()
        
    sys.stderr.write('  %d entries from %d reads.\n'%(cnt, len(mbByRead)))
    
    return mbByRead
    
def parseReads(fn):
    if fn.endswith('.gz'):
        import gzip
        f = gzip.open(fn)
    else:
        f = open(fn)
        
    cnt, readSeqs, ri2ci, seq = 0, {}, {}, ''

    sys.stderr.write('Parsing %s ... '%fn)
    for l in f:
        if l[0] == '>':
            if seq:
                readSeqs[rid] = seq
                seq = ''
            cid, rid = l[1:].split()[0].split('.')
            ri2ci[rid] = cid
            cnt += 1
        else: seq += l[:-1]

    if seq: readSeqs[rid] = seq
            
    sys.stderr.write('%d entries.\n'%cnt)

    return readSeqs, ri2ci

def processByRead(mbByRead, readSeqs, ri2ci):
    readIds = mbByRead.keys()
    readIds.sort
    chrs = {}
    for readId in readIds:
        ci = ri2ci.get(readId, None) #njc don't we already have circle id???

        readSeq = readSeqs[readId]
        readLength = len(readSeq)

        open(needleTmpFile+'.1', 'w').write('>read_needle_tmp\n%s\n'%readSeq)

        dbs = {'+': open(needleTmpFile+'_p.2', 'w'), '-': open(needleTmpFile+'_m.2', 'w')}

        hits = mbByRead[readId]
        pData = [None]*len(hits)
        for x in range(len(hits)):
            chrId, strand, circleId, readId, chrStart, readStart, chrEnd, readEnd, scoreD0 = hits[x]
            if not ci or ci != circleId:
                sys.stderr.write('Warning: Problem with circle id: %s, %s --- skipping.'%(str(ci), circleId))
                continue

            chrStart, readStart, chrEnd, readEnd = int(chrStart), int(readStart), int(chrEnd), int(readEnd) 

            # get seq and info
            if chrId not in chrs:
                # orginal code enforced a prefix of 'chr' for chrId. probably better not to.
                chrs[chrId] = mmapChr(chrId, chrIdToFile[chrId])
            mmc = chrs[chrId]
            
            # this computation is a little strange... the abs probably should have an extra +1, and is the comparison really <= or <?
            delta = int(config.expectedGapSpanMultiplier*readLength - abs(readEnd-readStart))
            if delta <= 1: delta = 0
            end = min(mmc.numSyms, chrEnd + delta)
            start = max(1, chrStart - delta)
            pData[x] = (chrId, start)

            try:   chrSeq = mmc.getSeq(start, end)
            except Exception, e:
                sys.stderr.write('Warning: cannot extract [%d, %d] for %s: %s\n'%(start, end, chrId, e))
                continue
            dbs[strand].write('>db_%d\n%s\n'%(x, chrSeq))

        for s in ['+', '-']:
            doIt = dbs[s].tell() != 0
            dbs[s].close()
            if doIt:
                for x, startPos, stopPos, identNTs, percentIdent in runWater(needleTmpFile+'.1', dbs[s].name, s):
                    print '\t'.join([str(f) for f in [ci, readId, pData[x][0], pData[x][1]+int(startPos)-1, pData[x][1]+int(stopPos)-1, s, percentIdent, identNTs]])

dbSeqTag = '\n# 2:'
dbSeqIdRe = re.compile(r'# 2: (db_\d+)\n') # this has to match the prefix above
identLineTag = '\n# Identity:'
identLineRe = re.compile(r'# Identity:\s+(\d+)/(\d+) \( *(\d+\.\d+)%\)\n')
def runWater(readFile, chrFragDBFile, strand):
    cmd = ' '.join([config.waterExecutable,
                    readFile,
                    chrFragDBFile,
                    config.waterMBRefineOptions,
                    '-datafile '+config.waterMBRefineDatafile,
                    'stdout'])
    if strand == '-': cmd += ' -sreverse1 TRUE'

    p = subprocess.Popen(cmd, shell = True, stdout=subprocess.PIPE)

    # we could recode this to accept streaming data from the water
    # process (using, say, a generator), but the water run should be
    # quick.
    waterOut = []
    for r in p.stdout.read().split('# Aligned_sequences:')[1:]:
        try: # to get id of db sequence
            x = r.index(dbSeqTag)+1
            dbSeqId = dbSeqIdRe.search(r[x:]).group(1)
        except:
            print >>stdE, 'Could not find db sequence id in "%s".'%(r[x:r.index('\n', x)])
            continue
        try: # to get identity line info
            x = r.index(identLineTag, x)+1
            identNTs, alignLen, percentIdent = identLineRe.search(r[x:]).groups()
        except:
            print >>stdE, 'Could not find Identity info in "%s".'%(r[x:r.index('\n', x)])
            continue
        try: # to find first part of alignment display
            x = r.index('\n'+dbSeqId, x)+1
            startPos = r[x:r.index('\n', x)].split()[1]
        except:
            print >>stdE, 'Could not find start of alignment info in "%s".'%(r[x:r.index('\n', x)])
            continue
        try: # to find last part of alignment display
            x = r.rindex('\n'+dbSeqId)+1
            stopPos = r[x:r.index('\n', x)].split()[3]
        except:
            print >>stdE, 'Could not find end of alignment info in "%s".'%(r[x:r.index('\n', x)])
            continue

        waterOut.append((int(dbSeqId.split('_')[-1]), startPos, stopPos, identNTs, percentIdent))
    p.wait()
    return waterOut

#-- main
# infile and only one (optional) argument: path to config data file. everything else is in there.
ops = optparse.OptionParser()
ops.description = 'Refine megablast hits using water.\n  ==> does not as yet consider reads mapping over the origin (can be easily fixed later)'
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.set_usage('%prog [options] megablast_hits reads')

oArgs, pArgs = ops.parse_args()

if len(pArgs) != 2:
    ops.print_help()
    sys.exit(1)

mbHits, reads = pArgs[:2]

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.

if config.PythonUtilitiesPath: sys.path += config.PythonUtilitiesPath
from getSeq import mmapChr

myPid = os.getpid()

needleTmpFile = '/tmp/needle_tmp_file.%d'%myPid

chrIdToFile = {}
for l in open(config.chrIdToFileMap):
    chrId, fn = l[:-1].split('<<>>')
    if chrId in chrIdToFile and fn != chrIdToFile[chrId]:
        print >>stdE, 'warning: %s maps to more than one file. Check file map: "%s".'%(chrId, config.chrIdToFileMap)
    chrIdToFile[chrId] = fn

print 'CIRCLE_ID\tREAD_ID\tCHR\tSTART\tEND\tSTRAND\tPERC_SEQID_UNWEIGHED\tLENGTH'
readSeqs, ri2ci = parseReads(reads)
mbByRead = parseMegablastOutput (mbHits)
processByRead(mbByRead, readSeqs, ri2ci)

# clean up
os.system('rm -f %s*'%needleTmpFile)
