#!/usr/bin/env python

import gc, imp, optparse, os, re, subprocess, sys

gc.disable()
stdE = sys.stderr

def runMegablast(infile):
    cmd = ' '.join([config.megablastExecutable,
                    '-d', config.megablastDatabase,
                    '-i', infile,
                    config.megablastOptions])

    print >>stdE, cmd+'\nRunning&Parsing ... '

    # regular expression for parsing -D 0 output from megablast with JK's inputs.
    # sample: '10'=='-000022_1874_3728.000022_1874_3728_B' (36101063 39 36101097 6) 3
    d0re = re.compile(r'\'([^\']+)\'==\'([-+])([^\.]+)\.([^\']+)\' \((\d+) (\d+) (\d+) (\d+)\) (\d+)')

    scoresByReadId = {}

    # pipeline megablast with output processing
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    for l in p.stdout:
        l = l[:-1]
        m = d0re.match(l)
        if not m:
            print >>stdE, 'Don\'t understand: '+l
            continue
        chrId, strand, circleId, readId, chrStart, readStart, chrEnd, readEnd, scoreD0 = m.groups()
        preScore = abs(float(readStart)-float(readEnd))+1 - 0.8*float(scoreD0)
        scoresByReadId.setdefault(readId, {}).setdefault(preScore, []).append(l)
        if len(scoresByReadId[readId]) > 4*config.max_scores_retained_per_seq:
            ks = scoresByReadId[readId].keys()
            ks.sort(reverse=True)
            scoresByReadId[readId] = dict([(s, scoresByReadId[readId][s]) for s in ks[:config.max_scores_retained_per_seq]])
    p.wait()

    # final trim
    for readId, hitsByScore in scoresByReadId.iteritems():
        scores = hitsByScore.keys()
        scores.sort(reverse=True)
        kept, trimmed = 0, {}
        for s in scores:
            trimmed[s] = hitsByScore[s][:config.max_duplicates_per_score]
            kept += len(trimmed[s])
            if kept >= config.max_scores_retained_per_seq: break
        scoresByReadId[readId] = trimmed

    return scoresByReadId

def output(scoresByReadId):
    print >>stdE, 'Output ... '

    readIds = scoresByReadId.keys()
    readIds.sort()
    for ri in readIds:
        hitsByScore = scoresByReadId[ri]
        scores = hitsByScore.keys()
        scores.sort(reverse=True)
        for s in scores:  print '\n'.join(hitsByScore[s])

#-- main

# infile and only one (optional) argument: path to config data file. everything else is in there.
ops = optparse.OptionParser()
ops.description = 'Run megablast over reads.'
ops.add_option('-c', '--config', help='override default configuration using values in FILE', metavar='FILE')
ops.set_usage('%prog [options] file_with_linker_mapped_reads')

oArgs, pArgs = ops.parse_args()

if len(pArgs) != 1:
    ops.print_help()
    sys.exit(1)

infile = pArgs[0]

config = imp.load_source('configData', os.path.dirname(__file__)+'/configPepDefault.py') # establish defaults
if oArgs.config:
    config = imp.load_source('configData', oArgs.config)     # allow overrides.

s = runMegablast(infile)
output(s)

print >>stdE, 'DONE!'
