TString _rootFileName  = "chr21.dir/chr21.fa.root";
TString _spansFileName   = "Sample_data_SV_simulation/Paired_end_spans.txt";
TString _readsFileName = "Sample_data_SV_simulation/lengths_of_reads_with_44bp_linker.txt";
TString _endsFileName  = "Sample_data_SV_simulation/distribution_of_lengths_for_ends_after_linker_removal.txt";

void make_distr(TString rootFileName  = _rootFileName,
		TString spansFileName = _spansFileName,
		TString readsFileName = _readsFileName,
		TString endsFileName  = _endsFileName)
{
  TFile *file = new TFile(rootFileName);

  TH1 *his_gen = NULL, *his_org = NULL;

  his_gen = (TH1*)file->Get("h_frg_len");
  his_org = (TH1*)his_gen->Clone("h_org");
  getFragmentsLengthDistribution(spansFileName,his_org);
  drawHis(his_gen,his_org,"Fragments length","frgs_len");

  his_gen = (TH1*)file->Get("h_read_len");
  his_org = (TH1*)his_gen->Clone("h_org");
  getReadsLengthDistribution(readsFileName,his_org);
  drawHis(his_gen,his_org,"Reads length","reads_len");

  his_gen = (TH1*)file->Get("h_end_len");
  his_org = (TH1*)his_gen->Clone("h_org");
  getEndsLengthDistribution(endsFileName,his_org);
  //drawHis(his_gen,his_org,"Ends length","ends_len");
  drawHis(his_org,his_gen,"Ends length","ends_len");

  his_gen = (TH1*)file->Get("h_frg_start");
  drawHis(his_gen,NULL,"Fragments start","frgs_start");

  his_gen = (TH1*)file->Get("h_frg_end");
  drawHis(his_gen,NULL,"Fragments end","frgs_end");

  his_gen = (TH1*)file->Get("h_sv_start");
  drawHis(his_gen,NULL,"SVs start","svs_start");

  his_gen = (TH1*)file->Get("h_sv_end");
  drawHis(his_gen,NULL,"SVs end","svs_end");

  his_gen = (TH1*)file->Get("h_sv_len");
  drawHis(his_gen,NULL,"SVs length","svs_len");

  TH1 *h_arr[10];
  h_arr[0] = (TH1*)file->Get("h_sig_null");
  h_arr[1] = (TH1*)file->Get("h_sig_1");
  h_arr[2] = (TH1*)file->Get("h_sig_2");
  h_arr[3] = (TH1*)file->Get("h_sig_3");
  h_arr[4] = (TH1*)file->Get("h_sig_4");
  h_arr[5] = (TH1*)file->Get("h_sig_5");
  h_arr[6] = (TH1*)file->Get("h_sig_6");
  h_arr[7] = (TH1*)file->Get("h_sig_7");
  h_arr[8] = (TH1*)file->Get("h_sig_8");
  h_arr[9] = (TH1*)file->Get("h_sig_9");


  TCanvas *canv = new TCanvas("canv","canv",1272,949);
  h_arr[0]->Draw();
  for (int i = 0;i < 10;i++) {
    TH1 *his = h_arr[i];
    his->Scale(1./his->Integral());
    if (i == 0) his->Draw();
    else        his->Draw("same");
    if (i == 0)      his->SetLineColor(kBlack);
    else if (i == 1) his->SetLineColor(kRed);
    else if (i == 2) his->SetLineColor(kYellow);
    else if (i == 3) his->SetLineColor(kBlue);
    else if (i == 4) his->SetLineColor(kGreen);
    else if (i == 5) his->SetLineColor(kGray);
    else if (i == 6) his->SetLineColor(kPink);
    else if (i == 7) his->SetLineColor(kOrange);
    else if (i == 8) his->SetLineColor(kMagenta);
    else if (i == 9) his->SetLineColor(kCyan);
    his->SetLineWidth(3);
  }
  h_arr[0]->GetXaxis()->SetTitle("Homopolymer size");
  h_arr[0]->GetYaxis()->SetTitle("Signal frequency");
  h_arr[0]->GetYaxis()->SetTitleOffset(1.2);
  canv->Print("error_model.jpg","jpg");
}

void drawHis(TH1 *his_gen,TH1 *his_org,TString x_title,TString fileName)

{
  if (!his_gen) return;

  TCanvas *canv = new TCanvas("canv","canv",1272,949);

  if (his_org) his_gen->Scale(1./his_gen->Integral());
  his_gen->Draw();
  his_gen->SetLineColor(kBlue);
  his_gen->SetLineWidth(3);
  his_gen->GetXaxis()->SetTitle(x_title);

  if (his_org) {
    his_org->Scale(1./his_org->Integral());
    his_org->Draw("same");
    his_org->SetLineColor(kGreen);
    his_org->SetLineWidth(3);
  }

  canv->Print(fileName + ".jpg","jpg");
}

void getFragmentsLengthDistribution(TString fileName,TH1 *his)
{
  his->SetName("h_frgs_length");
  his->SetTitle("Fragments length");
  his->Reset();

  ifstream in(fileName.Data());

  int len;
  while (!in.eof()) {
    in>>len;
    his->Fill(len);
  }

  in.close();
}

void getReadsLengthDistribution(TString fileName, TH1 *his)
{
  his->SetName("h_reads_length");
  his->SetTitle("Reads length");
  his->Reset();

  ifstream in(fileName.Data());

  int len;
  while (!in.eof()) {
    in>>len;
    his->Fill(len);
  }

  in.close();
}

void getEndsLengthDistribution(TString fileName,TH1 *his)
{
  his->SetName("h_ends_length");
  his->SetTitle("Ends length");
  his->Reset();

  ifstream in(fileName.Data());

  int len;
  while (!in.eof()) {
    in>>len;
    his->Fill(len);
  }

  in.close();
}


