#ifndef __GENOMESHIFTER_HH__
#define __GENOMESHIFTER_HH__

#include "SVEvent.hh"

class GenomeShifter
{
  // Constructors and detructores
public:
  GenomeShifter(SVEvent **svs,int n_events) : start_(NULL),
					      shift_(NULL),
					      n_events_(0)
  {
    if (n_events <= 0) return;
    SVEvent **tmp = new SVEvent*[n_events];
    for (int i = 0;i < n_events;i++) tmp[i] = svs[i];

    sortEvents(tmp,n_events);
    
    start_ = new int[n_events];
    shift_ = new int[n_events];

    n_events_ = 0;
    for (int i = 0;i < n_events;i++) {
      SVEvent *event = tmp[i];
      if (event->isInsertion()) {
	start_[n_events_] =  event->getStart();
	shift_[n_events_] = -event->getLength();
      } else if (event->isDeletion()) {
	start_[n_events_] = event->getStart() - 1;
	shift_[n_events_] = event->getLength();
      } else continue; // Inversions
      n_events_++;
    }

    delete[] tmp;
  }

  ~GenomeShifter()
  {
    delete[] start_;
    delete[] shift_;
  }

  // Internal arrays
private:
  int *start_,*shift_;
  int n_events_;

public: // Function to calculate shift
  void shift(int &v1,int &v2,int &v3,int &v4)
  {
    for (int i = 0;i < n_events_++;i++) {
      bool stop = true;
      if (v1 > start_[i]) {
	v1 += shift_[i];
	if (v1 <= start_[i]) v1 = -1;
	stop = false;
      }
      if (v2 > start_[i]) {
	v2 += shift_[i];
	if (v2 <= start_[i]) v2 = -1;
	stop = false;
      }
      if (v3 > start_[i]) {
	v3 += shift_[i];
	if (v3 <= start_[i]) v3 = -1;
	stop = false;
      }
      if (v4 > start_[i]) {
	v4 += shift_[i];
	if (v4 <= start_[i]) v4 = -1;
	stop = false;
      }
      if (stop) break;
    }
  }

  void shift(int &v1,int &v2)
  {
    for (int i = 0;i < n_events_++;i++) {
      bool stop = true;
      if (v1 > start_[i]) {
	v1 += shift_[i];
	if (v1 <= start_[i]) v1 = -1;
	stop = false;
      }
      if (v2 > start_[i]) {
	v2 += shift_[i];
	if (v2 <= start_[i]) v2 = -1;
	stop = false;
      }
      if (stop) break;
    }
  }

private:  
  void sortEvents(SVEvent **svs,int n_events)
  {
    for (int i1 = 0;i1 < n_events;i1++)
      for (int i2 = i1 + 1;i2 < n_events;i2++)
	if (svs[i2]->getStart() < svs[i1]->getStart()) {
	  SVEvent *tmp = svs[i1];
	  svs[i1] = svs[i2];
	  svs[i2] = tmp;
	}
  }
  
};


#endif
