#include "LengthDistribution.hh"

LengthDistribution::LengthDistribution(int upRange) :
  lowRange_(0), upRange_(1), max_(1), distr_(NULL)
{
  if (upRange <= 0)
    cerr<<"WARNING: Zero or negative value for distribution length."<<endl;
  else upRange_  = upRange;
  
  int n = upRange_ - lowRange_ + 1;
  distr_ = new int[n];
  for (int i = 0;i < n;i++) distr_[i] = 0;
  
  distr_[upRange] = 1;
}

LengthDistribution::LengthDistribution(TString fileName) :
  lowRange_(0), upRange_(1), max_(1), distr_(NULL)
{
  if (!fileName) {
    cerr<<"ERROR: No filename given."<<endl;;
    return;
  }

  ifstream in(fileName.Data());
  if (!in.is_open()) {
    cerr<<"ERROR: Cann't open file '"<<fileName<<"'."<<endl;
  } else {

    lowRange_ =  2147483647;
    upRange_  = -2147483647;

    int len;
    TString tmp;
    while (!in.eof()) {
      in>>tmp;
      if (tmp.Length() <= 0) continue;
      if (!tmp.IsDigit()) {
	cerr<<"Integer is expected instead of '"<<tmp<<"'."<<endl;
	cerr<<"Aborting reading distribution."<<endl;
	break;
      }
      len = atoi(tmp);
      if (len < lowRange_)     lowRange_ = len;
      else if (len > upRange_) upRange_  = len;
    }
    in.close();
  }

  int n = upRange_ - lowRange_ + 1;
  distr_ = new int[n];
  for (int i = 0;i < n;i++) distr_[i] = 0;

  in.open(fileName.Data());
  if (in.is_open()) {
    int len;
    while (!in.eof()) {
      in>>len;
      fill(len);
    }
    in.close();
  }
}

LengthDistribution::~LengthDistribution()
{
  delete distr_;
}

int LengthDistribution::fill(int val)
{
  if (val >= lowRange_ && val <= upRange_) {
    int bin = val - lowRange_;
    distr_[bin]++;
    if (distr_[bin] > max_) max_ = distr_[bin];
  }
}

int LengthDistribution::generateLength(bool skip_zero)
{
  int ret = int(gRandom->Uniform(lowRange_,upRange_ + 1.0));
  int val = int(gRandom->Uniform(max_ + 1.0));
  while ((skip_zero && distr_[ret - lowRange_] == 0) ||
	 val > distr_[ret - lowRange_]) {
    ret = int(gRandom->Uniform(lowRange_,upRange_ + 1.0));
    val = int(gRandom->Uniform(max_ + 1.0));
  }
  return ret;
}
