#ifndef __SVEVENT_HH__
#define __SVEVENT_HH__

class SVEvent : public Region
{
  // Constructors and detructores
private:
  SVEvent(int start,int end) : Region(start,end)
  {
    insert_len_  = 0;
  }

public:
  ~SVEvent ()
  {}

  // For insertion
private:
  int insert_len_;
public:
  int getLength()
  { 
    if (type_ == INSERTION_TYPE) return insert_len_;
    else                         return Region::getLength();
  }

  // Event constructors
public:
  static SVEvent *makeInsertion(int location,int len)
  {
    SVEvent *ret = new SVEvent(location,location);
    ret->setInsertion(len);
    return ret;
  }
  static SVEvent *makeDeletion(int start,int end)
  {
    SVEvent *ret = new SVEvent(start,end);
    ret->setDeletion();
    return ret;
  }
  static SVEvent *makeInversion(int start,int end)
  {
    SVEvent *ret = new SVEvent(start,end);
    ret->setInversion();
    return ret;
  }

  // Event generator
public:
  static SVEvent *generateEvent(int g_start,int g_end,int len,int type)
  {
    SVEvent *ret = NULL;
    if (type == DELETION_TYPE) {
      ret = generateForRegion(g_start,g_end,len);
      ret->setDeletion();
    } else if (type == INSERTION_TYPE) {
      ret = generateForPoint(g_start,g_end,len);
      ret->setInsertion(len);
    } else if (type == INVERSION_TYPE) {
      ret = generateForRegion(g_start,g_end,len);
      ret->setInversion();
    } else {
      cerr<<"Unknown event type '"<<type<<"'."<<endl;
    }
    return ret;
  }
private: static SVEvent *generateForRegion(int g_start,int g_end,int len)
  {
    int start = -1, end = -1, g_len = g_end - g_start + 1, half = len>>1;
    while (!(start >= g_start && start <= g_end &&
	     end   >= g_start && end   <= g_end)) {
      double rand = gRandom->Rndm();
      int mean    = g_start + int(rand*g_len);
      start   = mean - half;
      end     = start + len - 1;
    }

    SVEvent *ret = new SVEvent(start,end);
    return ret;
  }
private : static SVEvent *generateForPoint(int g_start,int g_end,int len)
  {
    int g_len    = g_end - g_start + 1;
    double rand  = gRandom->Rndm();
    int    start = g_start + int(rand*g_len);
    int    end   = start;
    SVEvent *ret = new SVEvent(start,end);
    return ret;
  }
  
public: static int readEvents(TString fileName,int *arr)
  {
    ifstream in(fileName.Data());
    if (!in.is_open()) {
      cerr<<"ERROR: Cann't open file '"<<fileName<<"'."<<endl;;
      return 0;
    }
    
    TString type,   n,   len;
    int     type_i, n_i, len_i;
    int n_lines = 0;
    while (!in.eof()) {
      in>>type;
      if (type.Length() <= 0) continue;
      TString tmp = type; tmp.ToLower();
      if (tmp == "deletion")       type_i = DELETION_TYPE;
      else if (tmp == "insertion") type_i = INSERTION_TYPE;
      else if (tmp == "inversion") type_i = INVERSION_TYPE;
      else {
	cerr<<"Unrecongnized SV type '"<<type<<"'."<<endl;
	cerr<<"Aborting reading SV events."<<endl;
	break;
      }
      if (in.eof()) break;
      in>>n;
      if (!n.IsDigit()) {
	cerr<<"Integer is expected insted of '"<<n<<"'."<<endl;
	cerr<<"Aborting reading SV events."<<endl;
	break;
      }
      n_i = atoi(n);
      if (in.eof()) break;
      in>>len;
      if (!n.IsDigit()) {
	cerr<<"Integer is expected insted of '"<<len<<"'."<<endl;
	cerr<<"Aborting reading SV events."<<endl;
	break;
      }
      len_i = atoi(len);
      arr[n_lines++] = type_i;
      arr[n_lines++] = n_i;
      arr[n_lines++] = len_i;
    }
    return n_lines/3;
  }

  // Type: insertion, deletion, inversion
private:
  static const int DELETION_TYPE  = 0;
  static const int INSERTION_TYPE = 1;
  static const int INVERSION_TYPE = 2;
  int type_;
  void setDeletion()         { type_ = DELETION_TYPE; }
  void setInsertion(int len) { type_ = INSERTION_TYPE; insert_len_ = len; }
  void setInversion()        { type_ = INVERSION_TYPE; }
public:
  bool isDeletion()  { return type_ == DELETION_TYPE; }
  bool isInsertion() { return type_ == INSERTION_TYPE; }
  bool isInversion() { return type_ == INVERSION_TYPE; }

  // List
private:
  SVEvent *next_;
public:
  SVEvent *getNext() { return next_; }
  void setNext(SVEvent *next) { next_ = next; }
};

ostream& operator << (ostream& os,SVEvent *sv)
{
  if (sv->isDeletion()) {
    os<<"deletion ";
  } else if (sv->isInsertion()) {
    os<<"insertion ";
  } else if (sv->isInversion()) {
    os<<"inversion ";
  }
  return os<<sv->getStart()<<" "<<sv->getEnd()<<" "<<sv->getLength();
}

#endif
