#ifndef __SEQUENATOR_HH__
#define __SEQUENATOR_HH__

//--- C/C++ includes ---
#include <iostream>
#include <math.h>

//--- ROOT includes ---
#include <TRandom3.h>

class Sequenator
{
private:
  static const int N_POLYMERS = 100;
  double cutoff[N_POLYMERS][2];
  double sig_sigma_[N_POLYMERS];

  // Constructors
public:
  Sequenator();
  Sequenator(double bg_mean,double bg_sigma,double sig_sigma);

private:
  double bg_mean_, bg_sigma_;
  double logmean_, logsigma_;
  void calcCutOffs();
  double getBGDistributionValue(double x);
  double getSignalDistributionValue(int n_bp,double x);

public:
  inline double signalSigma(int sig) { return sig_sigma_[sig]; }
  void setSignalSigmaLinear();
  void setSignalSigmaSquare();

  // Generating signal
public:
  inline double generate454BGValue()
  {
    return exp(gRandom->Gaus(logmean_,logsigma_));
  }
  inline double generate454SignalValue(int n_bp)
  {
    return gRandom->Gaus(n_bp,signalSigma(n_bp));
  }

  // Tracking errors
private:
  int poly_distr_[N_POLYMERS];
  int ins_distr_[N_POLYMERS];
  int del_distr_[N_POLYMERS];
  int sub_distr_[N_POLYMERS];
  void reset() ;
  void addSequenced(int n);
  void addInserted(int ind,int n);
  void addDeleted(int ind,int n);
  void addSubstituted(int ind,int n);

public:
  inline int getMaxPolymerSize()        { return N_POLYMERS; }
  inline int getNSequenced(int ind)     { return poly_distr_[ind]; }
  inline int getNInsertions(int ind)    { return ins_distr_[ind]; }
  inline int getNDeletions(int ind)     { return del_distr_[ind]; }
  inline int getNSubstitutions(int ind) { return sub_distr_[ind]; }

  // Sequencing
private:
  int    getNCalled(double val);
public:
  TString sequence454(TString seq);
  TString sequenceSolexa(TString seq);
};


#endif
