#!/usr/bin/perl

use warnings;
use strict;

my $usage = "The script checks whether SV event were correctly incorporated ";
$usage .= "into given genome.\n\n";
$usage .= "check_genomes.pl ref_genome sv_genome sv_list\n";

if (scalar(@ARGV) != 3) {
    print STDERR "Wrong number of arguments.\n\n";
    print $usage,"\n";
    exit;
}

my @svs = &readEvents($ARGV[2]);
if (scalar(@svs) == 0) {
    print STDERR "No event read/found. Exiting.\n";
    exit;
}

#print @svs,"\n";

if (!open(REF_FILE,$ARGV[0])) {
    print STDERR "Can't open file '",$ARGV[0],"'.\n";
    return;
}
if (!open(SV_FILE,$ARGV[1])) {
    print STDERR "Can't open file '",$ARGV[1],"'.\n";
    return;
}

my $line = <REF_FILE>; $line = <SV_FILE>;
my $ref_genome = <REF_FILE>; chomp($ref_genome);
my $sv_genome  = <SV_FILE>; chomp($sv_genome);
my ($ref_start,$ref_end) = (1,length($ref_genome));
my ($sv_start, $sv_end)  = (1,length($sv_genome));
my ($n_ref,$n_sv,$n_same,$n_del,$n_ins,$n_inv) = (0,0,0,0,0,0);
my $event_index = 0;
my $event_type  = &getSVType($svs[$event_index]);
my $event_end   = &getSVEnd($svs[$event_index]);
my $event_start = &getSVStart($svs[$event_index]);
my $event_len   = &getSVLen($svs[$event_index]);
while ($ref_end > $ref_start && $sv_end > $sv_start) {
    if ($ref_end > $event_start && $event_type eq "insertion") {
	# insertions
	#print $svs[$event_index];
	my $n_comp = $event_start - $ref_start + 1;
	if (!&same(\$ref_genome,\$sv_genome,$n_comp)) {
	    print "Ref: ",$ref_start,"\n";
	    print "SV:  ",$sv_start,"\n";
	    exit;
	} else {
	    $n_same += $n_comp;
	    $ref_genome = substr($ref_genome,$n_comp);
	    $ref_start += $n_comp;
	    $sv_genome = substr($sv_genome,$n_comp);
	    $sv_start += $n_comp;
	}
	
	while (length($sv_genome) < $event_len) {
	    if (($line = <SV_FILE>)) {
		chomp($line);
		$sv_genome .= $line;
 		$sv_end += length($line);
	    } else { last; }
	}

	$sv_genome = substr($sv_genome,$event_len);
	$sv_start += $event_len;
	$n_ins += $event_len;
	

	$event_index++;
	$event_type  = &getSVType($svs[$event_index]);
	$event_end   = &getSVEnd($svs[$event_index]);
	$event_start = &getSVStart($svs[$event_index]);
	$event_len   = &getSVLen($svs[$event_index]);
    } elsif ($ref_end >= $event_start && $event_type ne "insertion") {
	# deletion or inversion
	#print $svs[$event_index];
	my $n_comp = $event_start - $ref_start;
	if (!&same(\$ref_genome,\$sv_genome,$n_comp)) {
	    print "Ref: ",$ref_start,"\n";
	    print "SV:  ",$sv_start,"\n";
	    exit;
	} else {
	    $n_same += $n_comp;
	    $ref_genome = substr($ref_genome,$n_comp);
	    $ref_start += $n_comp;
	    $sv_genome = substr($sv_genome,$n_comp);
	    $sv_start += $n_comp;
	}

	while ($ref_end <= $event_end) {
	    if (($line = <REF_FILE>)) {
		chomp($line);
		$ref_genome .= $line;
		$ref_end += length($line);
	    } else { last; }
	}
	if ($event_type eq "deletion") {
	    $ref_genome = substr($ref_genome,$event_len);
	    $ref_start += $event_len;
	    $n_del += $event_len;
	} else { # Inversion
	    my $inversion = reverse(substr($ref_genome,0,$event_len));
	    $inversion =~ tr/ATCGatcg/TAGCtagc/;
	    $ref_genome = $inversion.substr($ref_genome,$event_len);
	}
	$event_index++;
	$event_type  = &getSVType($svs[$event_index]);
	$event_end   = &getSVEnd($svs[$event_index]);
	$event_start = &getSVStart($svs[$event_index]);
	$event_len   = &getSVLen($svs[$event_index]);
	next;
    } 
    
    my ($len_ref,$len_sv) = (length($ref_genome),length($sv_genome));
    my $n_comp = $len_ref; if ($len_sv < $n_comp) { $n_comp = $len_sv; }
    if (!&same(\$ref_genome,\$sv_genome,$n_comp)) {
	print "Ref: ",$ref_start,"\n";
	print "SV:  ",$sv_start,"\n";
	exit;
    }
    $n_same += $n_comp;
    $ref_genome = substr($ref_genome,$n_comp);
    $ref_start += $n_comp;
    $line = <REF_FILE>;
    if ($line) {
	chomp($line);
	$ref_genome .= $line;
	$ref_end += length($line);
    }

    $sv_genome  = substr($sv_genome,$n_comp);
    $sv_start  += $n_comp;
    while ($sv_end - $sv_start < $ref_end - $ref_start) {
	$line = <SV_FILE>;
	if ($line) {
	    chomp($line);
	    $sv_genome .= $line;
	    $sv_end += length($line);
	} else { last; }
    }
}

print "Reference genome is ",$n_same + $n_del," nucleotides in length.\n";
print "SV        genome is ",$n_same + $n_ins," nucleotides in length.\n";
print "No inconsistensies found.\n";

exit;

sub same
{
    my ($s1,$s2,$len) = @_;
    my $str1 = substr($$s1,0,$len);
    my $str2 = substr($$s2,0,$len);
    if ($str1 ne $str2) {
	print "Found difference.\n";
	print "str1: ",$str1,"\n";
	print "str2: ",$str2,"\n";
	return 0;
    }
    return 1;
}

sub getSVType()
{
    my $line = shift;
    if (!$line) { return "???"; }
    my @els = split(/\s+/,$line);
    return $els[0];
}

sub getSVStart()
{
    my $line = shift;
    if (!$line) { return 1e+100; }
    my @els = split(/\s+/,$line);
    return $els[1];
}

sub getSVEnd()
{
    my $line = shift;
    if (!$line) { return 1e+100; }
    my @els = split(/\s+/,$line);
    return $els[2];
}

sub getSVLen()
{
    my $line = shift;
    if (!$line) { return 1e+100; }
    my @els = split(/\s+/,$line);
    return $els[3];
}

sub readEvents
{
    my $fileName = shift;
    if (!open(FILE,$fileName)) {
	print STDERR "Can't open file '",$fileName,"'.\n";
	return;
    }

    my %svs = ();

    my $line = "";
    while ($line = <FILE>) {
	my @els = split(/\s+/,$line);
	$svs{$els[1]} = $line;
    }
    
    my @ret = ();
    for my $start (sort {$a <=> $b} keys %svs) {
	push(@ret,$svs{$start});
    }
    return @ret;
}
